<?php

namespace ShopWP\API\Items;

use ShopWP\Transients;
use ShopWP\Messages;
use ShopWP\Utils;
use ShopWP\Utils\Data as Utils_Data;
use ShopWP\CPT;

if (!defined('ABSPATH')) {
    exit();
}

class Products extends \ShopWP\API
{

    public $DB_Settings_General;
    public $DB_Settings_Syncing;
    public $DB_Metafields;
    public $Shopify_API;
    public $Processing_Database;
    public $Admin_API_Shop;
    public $Storefront_Products;
    public $API_Items_Collections;
    public $Webhooks;
    public $Admin_Queries;
    public $Filesystem;
    public $Processing_Media;
    public $Data_Bridge;

    public function __construct(
        $DB_Settings_General,
        $DB_Settings_Syncing,
        $DB_Metafields,
        $Shopify_API,
        $Processing_Database,
        $Admin_API_Shop,
        $Storefront_Products,
        $API_Items_Collections,
        $Webhooks,
        $Admin_Queries,
        $Filesystem,
        $Processing_Media,
        $Data_Bridge
    ) {
        $this->DB_Settings_General          = $DB_Settings_General;
        $this->DB_Settings_Syncing          = $DB_Settings_Syncing;
        $this->DB_Metafields                = $DB_Metafields;
        $this->Shopify_API                  = $Shopify_API;
        $this->Processing_Database          = $Processing_Database;
        $this->Admin_API_Shop               = $Admin_API_Shop;
        $this->Storefront_Products          = $Storefront_Products;
        $this->API_Items_Collections        = $API_Items_Collections;
        $this->Webhooks                     = $Webhooks;
        $this->Admin_Queries                = $Admin_Queries;
        $this->Filesystem                   = $Filesystem;
        $this->Processing_Media             = $Processing_Media;
        $this->Data_Bridge                  = $Data_Bridge;

        add_action('shopwp_bulk_recheck_cron_hook', [$this, 'shopwp_bulk_recheck_cron_hook_stuff']);
        add_action('shopwp_auto_sync_cron_callback', [$this, 'shopwp_auto_sync_cron_callback_handle']);

    }

    public function handle_expire_sync($request) {

        $cancel_auto_sync = $request->get_param('cancelAutoSync');

        $this->DB_Settings_Syncing->set_current_syncing_step_text(__('Finalizing ...', 'shopwp'));
        $this->DB_Settings_Syncing->toggle_syncing(0);
        
        return $this->DB_Settings_Syncing->expire_sync($cancel_auto_sync);
        
    }

    /*

	Gets published product ids as a URL param string

	*/
    public function get_published_product_ids_as_param($current_page)
    {
        $product_ids = $this->DB_Settings_Syncing->get_published_product_ids();

        if (empty($product_ids)) {
            return false;
        }

        $limit = $this->DB_Settings_General->get_items_per_request();

        return $this->Shopify_API->create_param_ids(
            $product_ids,
            $limit,
            $current_page
        );
    }

    /*

	Gets products by page

	*/
    public function get_products_per_page($current_page)
    {

        $product_ids_comma_string = $this->get_published_product_ids_as_param($current_page);
        $limit = $this->DB_Settings_General->get_items_per_request();

        $response = $this->Shopify_API->get_products_per_page(
            $product_ids_comma_string,
            $limit
        );

        return $this->Shopify_API->pre_response_check($response);
    }

    public function normalize_filter_data($data) {
        return array_map(function($item) {
            return $item->node;
        }, $data->edges);
    }


    public function handle_get_all_product_tags($request) {

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');

        if ($cached_enabled) {
            $cached_tags = Transients::get('shopwp_all_tags');

            if (!empty($cached_tags)) {
                return \wp_send_json_success($cached_tags);
            }
        }

        $tags = $this->Admin_API_Shop->get_tags();

        if (is_wp_error($tags)) {
            return $this->handle_response($tags);
        }

        $tags = $this->normalize_filter_data($tags);

        if ($cached_enabled) {
            Transients::set('shopwp_all_tags', $tags, $data_cache_length);
        }

        return \wp_send_json_success($tags);
    }

    public function handle_get_all_product_vendors($request) {

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');

        if ($cached_enabled) {
            $cached_vendors = Transients::get('shopwp_all_vendors');

            if (!empty($cached_vendors)) {
                return \wp_send_json_success($cached_vendors);
            }
        }

        $vendors = $this->Admin_API_Shop->get_vendors();

        if (is_wp_error($vendors)) {
            return $this->handle_response($vendors);
        }

        $vendors = $this->normalize_filter_data($vendors);

        if ($cached_enabled) {
            Transients::set('shopwp_all_vendors', $vendors, $data_cache_length);
        }

        return \wp_send_json_success($vendors);

    }

    public function handle_get_all_product_types($request) {

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');

        if ($cached_enabled) {

            $cached_types = Transients::get('shopwp_all_types');

            if (!empty($cached_types)) {
                return \wp_send_json_success($cached_types);
            }

        }

        $types = $this->Admin_API_Shop->get_product_types();

        if (is_wp_error($types)) {
            return $this->handle_response($types);
        }

        $types = $this->normalize_filter_data($types);

        if ($cached_enabled) {
            Transients::set('shopwp_all_types', $types, $data_cache_length);
        }

        return \wp_send_json_success($types);
    }

    /*
    
    Public API Method
    
    */
    public function get_products($params) {

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');

        if ($cached_enabled) {

            $hash = Utils::hash($params, true);

            $cached_query = Transients::get('shopwp_query_' . $hash);

            $cached_query = \maybe_unserialize($cached_query);

            if (!empty($cached_query)) {
                return $cached_query;
            }
        }

        $final_params = $this->public_api_default_values($params);

        $result = $this->Storefront_Products->api_get_products($final_params, $final_params['schema']);

        if (is_wp_error($result)) {
            return $result;
        }

        if ($cached_enabled) {
            Transients::set('shopwp_query_' . $hash, \maybe_serialize($result), $data_cache_length);
        }

        return $result;

    }

    public function get_product($params) {

        $query_params = [
            'storefront_id'  => $this->create_product_storefront_id($params),
            'language'       => !empty($params['language']) ? strtoupper($params['language']) : strtoupper($this->DB_Settings_General->col('language_code', 'string')),
            'country'        => !empty($params['country']) ? strtoupper($params['country']) : strtoupper($this->DB_Settings_General->col('country_code', 'string')),
        ];

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');
        $schema = !empty($params['schema']) ? $params['schema'] : false;

        $cache_key = md5($query_params['storefront_id'] . $query_params['language'] . $query_params['country'] . $schema);

        if (empty($query_params['storefront_id'])) {
            return Utils::wp_error(__('No storefront id provided', 'shopwp'));
        }

        if ($cached_enabled) {

            $cached_query = \maybe_unserialize(Transients::get('shopwp_query_' . $cache_key));

            if (!empty($cached_query)) {
                return $cached_query;
            }
        }

        $result = $this->Storefront_Products->api_get_product_by_id($query_params, $schema);

        if (is_wp_error($result)) {
            return $result;
        }

        if ($cached_enabled) {
            Transients::set('shopwp_query_' . $cache_key, \maybe_serialize($result), $data_cache_length);
        }

        return $result;

    }

    public function format_collection_ids($collection_ids) {
        return array_map(function($collection_id) {

            if (is_array($collection_id)) {
                return 'gid://shopify/Collection/' . $collection_id['id'];
            } else {
                return 'gid://shopify/Collection/' . $collection_id;
            }
            
        }, $collection_ids);
    }

    public function get_all_products_from_collection_id($query_params, $custom_schema = false, $has_next_page = true, $total_products = []) {

        // called during processing
        $result = $this->Storefront_Products->api_get_products_from_collection_id($query_params, $custom_schema);

        if (is_wp_error($result)) {
            return $result;
        }

        if (empty($result) || empty($result[0])) {
            return $total_products;
        }

        $products = $result[0]->products->edges;

        $only_products = [];

        foreach ($products as $product) {
            $only_products[] = $product->node;
        }

        $total_products = array_merge($total_products, $only_products);

        $last_cursor_id = $this->Shopify_API->get_last_cursor($products);

        $has_next_page = $this->Shopify_API->has_next_page($result[0]->products);

        if ($has_next_page) {
            $query_params['cursor'] = $last_cursor_id;

            return $this->get_all_products_from_collection_id($query_params, $custom_schema, $has_next_page, $total_products);

        } else {
            return $total_products;
        }
    }

    public function get_all_products($query_params, $custom_schema, $has_next_page = false, $total_products = []) {

        $result = $this->Storefront_Products->api_get_products($query_params, $custom_schema);

        if (is_wp_error($result)) {
            return $result;
        }

        if (empty($result)) {
            return $total_products;
        }

        $products = $result->edges;

        if (empty($products)) {
            return [];
        }

        $total_products     = array_merge($total_products, $products);
        $last_cursor_id     = $this->Shopify_API->get_last_cursor($products);
        $has_next_page      = $this->Shopify_API->has_next_page($result);

        if ($has_next_page) {
            $query_params['cursor'] = $last_cursor_id;

            return $this->get_all_products($query_params, $custom_schema, $has_next_page, $total_products);

        } else {

            return array_values(array_map(function($product) {
                return $product->node;
            }, $total_products));
        }

    }

    public function get_all_products_from_collection_ids($collection_ids, $query_params, $custom_schema = false) {
    
        $final_all_products = [];

        foreach ($collection_ids as $collections_id) {

            $query_params['ids'] = [$collections_id];

            $result = $this->get_all_products_from_collection_id($query_params, $custom_schema);

            if (is_wp_error($result)) {
                $final_all_products = $result;
                break;
            }

            if ($result) {
                $final_all_products = array_merge($final_all_products, $result);
            }
        }

        if (is_array($final_all_products)) {
            return array_values(array_unique($final_all_products, SORT_REGULAR));
        }

        return $final_all_products;
    }

    /*
    
    Public API Method
    
    */
    public function get_products_by_collection_ids($params)
    {
        if (empty($params) || empty($params['collection_ids']) || !is_array($params['collection_ids'])) {
            return [];
        }

        $cache_id = md5(implode("-", $params['collection_ids']));

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');

        if ($cached_enabled) {

            $cached_query = \maybe_unserialize(Transients::get('shopwp_query_' . $cache_id));

            if (!empty($cached_query)) {
                return $cached_query;
            }
        }
        

        $collection_ids = $this->format_collection_ids($params['collection_ids']);

        $query_params = $this->public_api_default_values($params);

        $all_products = $this->get_all_products_from_collection_ids($collection_ids, $query_params, $query_params['schema']);

        if (is_wp_error($all_products)) {
            return $all_products;
        }

        if ($cached_enabled) {
            Transients::set('shopwp_query_' . $cache_id, \maybe_serialize($all_products), $data_cache_length);
        }

        return $all_products;

    }

    public function create_product_storefront_id($params) {

        $prefix = 'gid://shopify/Product/';

        if (!is_array($params) | empty($params)) {
            return false;
        }

        if (isset($params['post_id'])) {
            $product_id = get_post_meta($params['post_id'], 'product_id', true);

            if (empty($product_id)) {
                return false;
            }

            $storefront_id = $prefix . $product_id;

        } else if (isset($params['product_id'])) {
            $storefront_id = $prefix . $params['product_id'];

        } else if (isset($params['storefront_id'])) {
            $storefront_id = $params['storefront_id'];

        } else {
            $storefront_id = false;
        }

        return $storefront_id;
        
    }

    public function handle_get_posts($request) {
        
        $data = $request->get_param('data');

        if (!isset($data['nonce'])) {
            return \wp_send_json_error(Utils::wp_error(__('No security token found, exiting.', 'shopwp')));
        }

        if (!\wp_verify_nonce($data['nonce'], 'wp_rest')) {
            return \wp_send_json_error(Utils::wp_error(__('Security token not valid, exiting.', 'shopwp')));
        }

        global $wpdb;

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');

        if ($cached_enabled) {
            $hash = Utils::hash($data, true);

            $cached_query = Transients::get('shopwp_query_' . $hash);

            $cached_query = \maybe_unserialize($cached_query);

            if (!empty($cached_query)) {
                return \wp_send_json_success($cached_query);
            }
        }

        $query = "SELECT " . $data['fields'] . " FROM {$wpdb->posts} WHERE post_type = '" . $data['post_type'] . "'";

        // A sql query to return all post titles
        $stuff = $wpdb->get_results($query, ARRAY_A );

        if ($cached_enabled) {
            Transients::set('shopwp_query_' . $hash, \maybe_serialize($stuff), $data_cache_length);
        }

        return \wp_send_json_success($stuff);
    }

    public function maybe_add_metafields_to_payload($result) {

        $is_syncing_metafields              = $this->DB_Settings_General->col('selective_sync_metafields', 'bool');
        $include_metafields_on_payload      = $this->DB_Settings_General->col('include_metafields_on_payload', 'bool');
        
        if ($is_syncing_metafields && $include_metafields_on_payload) {
            
            $all_metafields = $this->DB_Metafields->get_metafields();

            $result->edges = array_map(function($payload) use($all_metafields) {

                $result = str_replace('gid://shopify/Product/', '', $payload->node->id);

                $found = array_values(array_filter($all_metafields, function($metafield) use($result) {
                    return $metafield->productId === $result;
                }));

                $payload->node->metafields = $found;

                return $payload;

            }, $result->edges);

        }

        return $result;
    }

    public function handle_get_products($request) {

        $data = $request->get_param('data');

        if (empty($data)) {
            return \wp_send_json_error(Utils::wp_error(__('No data was found for get products, exiting.', 'shopwp')));
        }

        if (isset($data['shopState'])) {
            $shop_state = $data['shopState'];
        } else {
            $shop_state = false;
        }

        if (isset($data['queryParams'])) {
            $query_params = $data['queryParams'];
        } else {
            $query_params = false;
        }

        if (!empty($data['metafields'])) {
            $query_params['metafields'] = $data['metafields'];
        }

        \set_time_limit(0);

        $cached_enabled         = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length      = $this->DB_Settings_General->col('data_cache_length', 'int');

        if (empty($shop_state)) {
            $buyer_id = false;

        } else {
            $buyer_id = $shop_state['buyerIdentity'];
        }
        
        $query_params = $this->Storefront_Products->GraphQL->normalize_buyer_identity($buyer_id, $query_params);

        if ($cached_enabled) {
            $hash = Utils::hash($query_params, true);

            $cached_query = Transients::get('shopwp_query_' . $hash);

            $cached_query = \maybe_unserialize($cached_query);
            
            if (!empty($cached_query)) {
                return \wp_send_json_success($cached_query);
            }
        }

        $result = $this->Storefront_Products->api_get_products($query_params);

        if (is_wp_error($result)) {
            return $this->handle_response($result);
        }

        $result = $this->maybe_add_metafields_to_payload($result);

        $result = apply_filters('shopwp_products_payload', $result, $query_params['buyerIdentity']['language']);

        if ($cached_enabled) {
            Transients::set('shopwp_query_' . $hash, \maybe_serialize($result), $data_cache_length);
        }

        return \wp_send_json_success($result);

    }

    public function build_collection_id_obj($collection_title, $cached_collections_list, $found_key) {
        return [
            'label' => $collection_title,
            'id' => base64_encode('gid://shopify/Collection/' . $cached_collections_list[$found_key]->id)
        ];
    }

    public function adjust_query_params_for_products_by_collection_titles($query_params) {

        $supplied_titles = $query_params['collection_titles'];

        if (is_string($supplied_titles)) {
            $supplied_titles = [strtolower($supplied_titles)];
        } else {
            $supplied_titles = array_map('strtolower', $supplied_titles);
        }

        $cached_collections_list = maybe_unserialize(Transients::get('shopwp_all_collections'));

        if (empty($cached_collections_list)) {
            $cached_collections_list = $this->API_Items_Collections->handle_get_all_collections(false);
        }

        $new_stuff = array_filter(array_map(function($collection_title) use($cached_collections_list) {

            $cached_collection_titles_to_search = array_map('strtolower', array_column($cached_collections_list, 'title'));

            $found_key = array_search($collection_title, $cached_collection_titles_to_search);

            if ($found_key === false) {
                return false;
            }

            return $this->build_collection_id_obj($collection_title, $cached_collections_list, $found_key);

        }, $supplied_titles));

        return $new_stuff;

    }
    
    public function handle_get_products_from_collections($request) {

        $data = $request->get_param('data');

        if (empty($data)) {
            return \wp_send_json_error(Utils::wp_error(__('No data was found for get products from collections, exiting.', 'shopwp')));
        }

        if (isset($data['shopState'])) {
            $shop_state = $data['shopState'];
        } else {
            $shop_state = false;
        }

        if (isset($data['queryParams'])) {
            $query_params = $data['queryParams'];

            if (!empty($data['metafields'])) {
                $query_params['metafields'] = $data['metafields'];
            }
            
        } else {
            $query_params = false;
        }
        
        /*
        
        If the user is searching for products based on collection title, then
        we need to transform the data into ids. To do this, we need to first look 
        to see if they have a cached collections response. If they do, we can use that 
        to find the collection id. 

        If they don't, we need to fetch all the collections, cache them, and find the 
        id that way.
        
        */

        if (!empty($query_params['collection_titles'])) {

            $query_params['ids'] = $this->adjust_query_params_for_products_by_collection_titles($query_params);

            if (empty($query_params['ids'])) {
                return \wp_send_json_success([]);
            }

            $hash = Utils::hash($query_params, true);

        } else {
            $hash = Utils::hash($query_params, true);
        }

        $cached_enabled = $this->DB_Settings_General->col('enable_data_cache', 'bool');
        $data_cache_length = $this->DB_Settings_General->col('data_cache_length', 'int');

        if ($cached_enabled) {
            $cached_query = Transients::get('shopwp_query_' . $hash);

            $cached_query = \maybe_unserialize($cached_query);

            if (!empty($cached_query)) {
                return \wp_send_json_success($cached_query);
            }
            
        }

        $query_params['language'] = $shop_state['buyerIdentity']['language'];
        $query_params['country'] = $shop_state['buyerIdentity']['country'];

        $query_params = $this->Storefront_Products->GraphQL->normalize_buyer_identity($query_params, $query_params);

        if (!empty($query_params['query'])) {

            $query_params['first'] = (int)$query_params['first'];

            // Called from either shortcode or Render API
            $result = $this->Storefront_Products->api_get_products_from_collection_id($query_params);

            if (Utils::str_contains($query_params['query'], 'available_for_sale:true')) { 
                
                // filter out out of stock products
                $result = array_map(function($collection) {

                    return [
                        'id' => $collection->id,
                        'products' => [
                            'pageInfo' => [
                                'hasNextPage' => $collection->products->pageInfo->hasNextPage,
                                'hasPreviousPage' => $collection->products->pageInfo->hasPreviousPage
                            ],
                            'edges' => array_values(array_filter($collection->products->edges, function($product) {
                                return $product->node->availableForSale;
                            }))
                        ]
                    ];
                }, $result);
            }

        } else {

            // Called from Storefront collections filter
            $result = $this->get_all_products_from_collection_ids($query_params['ids'], $query_params);

            $result = [
                'nodes' => [[
                    'products' => [
                        'cursor' => '',
                        'edges' => $result,
                        'pageInfo' => [
                                'hasNextPage' => false,
                                'hasPreviousPage' => false
                            ]
                    ]
                ]]
            ];
        }

        if (is_wp_error($result)) {
            return $this->handle_response($result);
        }

        // If we land here, we're most likely fetching products from multiple collections. Need to combine them into one array
        if (count($result) > 1) {

            $stuff = [
                'pageInfo' => [
                    'hasNextPage' => false,
                    'hasPreviousPage' => false
                ],
                'nodes' => []
            ];

            foreach ($result as $item) {
                $stuff['nodes'] = \array_merge($stuff['nodes'], $item->products->edges);
            }

            $result = $stuff;

        }


        /*
        
        If user selected more than one collection
        
        */
        if (is_array($result)) {
        
            if (!empty($result['nodes'])) {

                $result = $result['nodes'];

                $p = new \stdClass();
                $p->pageInfo = [
                    'hasNextPage' => false,
                    'hasPreviousPage' => false
                ];

                $p->edges = apply_filters('shopwp_products_payload', $result, $shop_state['buyerIdentity']['language']);
                
                $p = $this->maybe_add_metafields_to_payload($p);

                $okok = [
                    [
                        'products' => $p
                    ]
                ];

                if ($cached_enabled) {
                    Transients::set('shopwp_query_' . $hash, \maybe_serialize($okok), $data_cache_length);
                }

                return \wp_send_json_success($okok);

            }

        }
        

        if (is_object($result[0])) {
            $result[0]->products = apply_filters('shopwp_products_payload', $result[0]->products, $shop_state['buyerIdentity']['language']);
            $result[0]->products = $this->maybe_add_metafields_to_payload($result[0]->products);

		} else {
            $result[0]['products'] = apply_filters('shopwp_products_payload', $result[0]['products'], $shop_state['buyerIdentity']['language']);
            $result[0]['products'] = $this->maybe_add_metafields_to_payload($result[0]['products']);
        }

        if ($cached_enabled) {
            Transients::set('shopwp_query_' . $hash, \maybe_serialize($result), $data_cache_length);
        }

        return \wp_send_json_success($result);
        
    }

    public function handle_sync_data($request) {

        $bulk_query = $this->DB_Settings_General->col('bulk_products_query', 'string');

        if (empty($bulk_query) || $bulk_query === '') {
            $bulk_query = '*';
        }

        $result = $this->sync_shopify_data($bulk_query);

        if (\is_wp_error($result)) {
            return \wp_send_json_error($result);

        } else {
            return \wp_send_json_success($result);
        }

    }































    public function get_queries_for_bulk_op($bulk_query, $selective_sync, $sync_by_collections = false) {

        if (!$sync_by_collections) {
            $sync_by_collections = $this->DB_Settings_General->sync_by_collections();
        }

        $products_query         = $this->get_products_query($bulk_query, $selective_sync, $sync_by_collections);

        $customers_query        = $this->Admin_Queries->get_customers_query($selective_sync);
        $orders_query           = $this->Admin_Queries->get_orders_query($selective_sync);

        return [
            'products'  => $products_query,
            'customers' => $customers_query,
            'orders'    => $orders_query
        ];
    }

    public function is_not_syncing_products($s) {
      return empty($s['products']) && empty($s['variants']) && empty($s['options']) && empty($s['tags']) && empty($s['images']) && empty($s['metafields']);
   }

   public function get_products_query($bulk_products_query, $selective_sync, $sync_by_collections) {

        $schema = $this->Admin_Queries->default_product_schema($selective_sync, $sync_by_collections);

        if (empty($schema)) {
            return '';
        }

        if ($this->is_not_syncing_products($selective_sync)) {

            if (!empty($selective_sync['collections'])) {

                $query = $this->Admin_Queries->get_collections_query($schema, $selective_sync, $sync_by_collections, false);

            } else {

                // User is syncing customers or orders
                $query = false;
            }

        } else {

            if (empty($sync_by_collections) ) {

                $final_q = '"' . $bulk_products_query . ' AND published_status:\'shopwp:visible\'"';

                $query = '
                    products(query: ' . $final_q . ') {
                        edges {
                            node {
                            ' . $schema . '
                            }
                        }
                    }
                ';

            } else {
                $query = $this->Admin_Queries->get_collections_query($schema, $selective_sync, $sync_by_collections);
            }
        }

        return $query;

   }

    /*

    Can be used as a third-party public API since we're manually turning on the sync

    $options: Array
        'is_cron'
        'selective_sync'
            - products
            - variants
            - options
            - tags
            - images
            - metafields 
            - customers 
            - orders 
            - collections

    */
    public function start_sync($query, $options = []) {

        $this->DB_Settings_Syncing->update_col('is_syncing', 1);
        $this->DB_Settings_Syncing->update_col('expire_sync_from_server', 1);

        return $this->sync_shopify_data($query, $options);
    }


    public function sync_shopify_data($bulk_query, $options = []) {
        
        $current_bulk_id            = $this->DB_Settings_Syncing->col('bulk_op_id', 'string');
        $sync_by_collections        = $this->DB_Settings_General->sync_by_collections();
        $current_bulk_operations    = $this->Webhooks->admin_api_get_current_bulk_operation();
        
        if (is_wp_error($current_bulk_operations)) {
            $this->DB_Settings_Syncing->log($current_bulk_operations->get_error_message(), 'error');

        } else {

            if (!empty($current_bulk_operations)) {
                
                if ($current_bulk_operations->status === 'RUNNING' || $current_bulk_operations->status === 'CANCELING') {

                    if ($current_bulk_operations->id !== $current_bulk_id ) {
                        
                        $msg_polling = __('Found an existing Shopify bulk operation in progress not created by ShopWP. Waiting for it to finish ...', 'shopwp');
                        
                        $this->DB_Settings_Syncing->log($msg_polling, 'warning');
                        $this->DB_Settings_Syncing->set_current_syncing_step_text($msg_polling);

                        if (!empty($options['is_cron'])) {

                            $this->Webhooks->maybe_schedule_cron('shopwp_bulk_recheck_cron_hook', 'shopwp_bulk_operation_recheck_interval');
                            
                            return $current_bulk_operations;
                        }

                    } else {
                        
                        $this->DB_Settings_Syncing->log(__('Canceling existing bulk operation: ', 'shopwp') . $current_bulk_id . ' ...');

                        $cancel_bulk_op_response = $this->Webhooks->admin_api_cancel_bulk_operation($current_bulk_id);

                        if (is_wp_error($cancel_bulk_op_response)) {

                            $this->DB_Settings_Syncing->log(__('Failed to cancel existing bulk operation: ', 'shopwp') . $current_bulk_id . ' ... ' . $cancel_bulk_op_response->get_error_message(), 'warning');

                            $this->DB_Settings_Syncing->log(__('Clearing old bulk operation id from database and continuing sync ...', 'shopwp'));
                            $this->DB_Settings_Syncing->update_col('bulk_op_id', '');

                        }

                    }

                }

            }
            
        }
        
        // At this point, no existing bulk ops found, or they have been completed already.
        $this->DB_Settings_Syncing->log(__('Starting new Shopify sync ...', 'shopwp'));
        $this->DB_Settings_Syncing->set_current_syncing_step_text(__('Connecting the bulk data webhook ...', 'shopwp'));

        $current_bulk_webhook_id = $this->DB_Settings_Syncing->col('bulk_op_webhook_subscription_id', 'string');

        if (!empty($current_bulk_webhook_id)) {

            $this->DB_Settings_Syncing->log(__('Removing old bulk data webhook ', 'shopwp') . $current_bulk_webhook_id . ' ...');

            $remove_old_sub_response = $this->Webhooks->admin_api_delete_bulk_webhook_subscription($current_bulk_webhook_id);
                        
            if (is_wp_error($remove_old_sub_response)) {

                // We can safely assume that a stale id exists at this point. Just need to reset and continue
                $this->DB_Settings_Syncing->log(__('Failed removing old bulk data webhook: ', 'shopwp') . $current_bulk_webhook_id . ' ... ' . $remove_old_sub_response->get_error_message(), 'warning');

                $this->DB_Settings_Syncing->log(__('Clearing old webhook id from database and continuing sync ...', 'shopwp'));
                
                $this->DB_Settings_Syncing->update_col('bulk_op_webhook_subscription_id', '');
            }

        } else {

            // While the webhook subscription may be empty, there could still be one connected. If this happens, we need cancel it and reconnect
            $all_webhook_subscriptions = $this->Webhooks->admin_api_get_all_webhook_subscriptions();

            if (is_wp_error($all_webhook_subscriptions)) {
                $this->DB_Settings_Syncing->log($all_webhook_subscriptions->get_error_message(), 'error');
            }

            if (!empty($all_webhook_subscriptions->edges)) {
                
                $only_bulk_webhooks = array_values(array_filter($all_webhook_subscriptions->edges, function($active_webhook) {
                    return $active_webhook->node->topic === 'BULK_OPERATIONS_FINISH';
                }));

                if (!empty($only_bulk_webhooks)) {
                    $webhook_to_remove = $only_bulk_webhooks[0]->node->id;

                    $remove_response = $this->Webhooks->admin_api_delete_bulk_webhook_subscription($webhook_to_remove);

                    if (is_wp_error($remove_response)) {
                        $this->DB_Settings_Syncing->log(__('Tried removing old webhook ', 'shopwp') . $webhook_to_remove . __(' but failed.', 'shopwp') . $remove_response->get_error_message(), 'error');
                    } else {
                        $this->DB_Settings_Syncing->log(__('Successfully removed old webhook, adding new one ...', 'shopwp'));
                    }

                }
            }
        }

        if (!$this->DB_Settings_Syncing->is_syncing()) {
            $this->DB_Settings_Syncing->expire_sync();
            return;
        }

        $callback_url = $this->DB_Settings_General->col('url_webhooks', 'string');
        $full_callback = $callback_url . '?shopify_bulk_webhook=true';

        $this->DB_Settings_Syncing->log(__('Connecting bulk data webhook to url: ', 'shopwp') . $full_callback . ' ...');

        $response_webhook_sub = $this->Webhooks->admin_api_create_bulk_webhook_subscription($full_callback);

        if (is_wp_error($response_webhook_sub)) {

            $this->DB_Settings_Syncing->log($response_webhook_sub->get_error_message(), 'error');
            $this->DB_Settings_Syncing->expire_sync();
            return;

        }

        if (!empty($response_webhook_sub->id)) {
            $this->DB_Settings_Syncing->log(__('Saving new webhook subscription id: ', 'shopwp') . $response_webhook_sub->id);
            $this->DB_Settings_Syncing->update_col('bulk_op_webhook_subscription_id', $response_webhook_sub->id);
        } else {
            $this->DB_Settings_Syncing->log(__('Tried creating new webhook but response was empty.', 'shopwp'), 'warning');
        }

        $this->DB_Settings_Syncing->set_current_syncing_step_text(__('Fetching data from Shopify, please wait ...', 'shopwp'));

        if (empty($options['selective_sync'])) {
            $selective_sync = $this->DB_Settings_General->selective_sync_status();

        } else {
            $selective_sync = $options['selective_sync'];
        }

        if (!empty($options['sync_by_collections'])) {
            $sync_by_collections = $options['sync_by_collections'];
        }

        $result = array_filter($selective_sync, function($data) {
            return !empty($data);
        });

        if (empty($result)) {
            $this->DB_Settings_Syncing->log(__('No Shopify data selected to sync. You must choose something to sync first. ', 'shopwp'), 'error');
            $this->DB_Settings_Syncing->expire_sync();
            return;
        }

        if (is_wp_error($bulk_query)) {
            $this->DB_Settings_Syncing->log($bulk_query->get_error_message(), 'error');
            $this->DB_Settings_Syncing->expire_sync();
            return;
        }

        $this->DB_Settings_Syncing->log('-----------------');
        $this->DB_Settings_Syncing->log(__('Fetching Shopify data. Please be patient as this may take awhile depending on how many products you\'re syncing. It\'s normal to wait up to 5 minutes for thousands of products. Syncing will resume automatically once the data is received.', 'shopwp'));
        $this->DB_Settings_Syncing->log('-----------------');
        
        $queries = $this->get_queries_for_bulk_op($bulk_query, $selective_sync, $sync_by_collections);

        $bulk_op_response = $this->Webhooks->admin_api_create_bulk_op($queries);

        if (is_wp_error($bulk_op_response)) {
            $this->DB_Settings_Syncing->log($bulk_op_response->get_error_message(), 'error');
            $this->DB_Settings_Syncing->expire_sync();
            return $bulk_op_response;
        }

        $this->DB_Settings_Syncing->update_col('bulk_op_id', $bulk_op_response->id);

        return $bulk_op_response;

    }

    public function handle_remove_synced_data($request) {

        $remove_all             = $request->get_param('removeAll');
        $check_clear_before     = $request->get_param('checkClearBeforeSyncing');

        $msg = __('Removing previously synced data ...', 'shopwp');

        $this->DB_Settings_Syncing->log($msg);
        $this->DB_Settings_Syncing->set_current_syncing_step_text($msg);
        
        // Kick off the async data deletion 
        $this->Processing_Database->delete_synced_data($remove_all, $check_clear_before);

        return true;

    }

    public function handle_remove_synced_images($request) {

        $msg = __('Removing synced images from WordPress media library ...', 'shopwp');

        $this->DB_Settings_Syncing->log($msg);
        $this->DB_Settings_Syncing->set_current_syncing_step_text($msg);
        
        // Kick off the async images deletion 
        $this->Processing_Media->delete_synced_images();

        return true;

    }

    public function shopwp_auto_sync_cron_callback_handle() {

        if ($this->DB_Settings_Syncing->is_syncing()) {
            $this->DB_Settings_Syncing->log(__('Already syncing, waiting until current sync is finished ...', 'shopwp'));
            return true;

        } else {
            $this->DB_Settings_Syncing->toggle_syncing(1);
        }

        // Clear previous log file 
        $Filesystem             = $this->Filesystem->bootstrap_filesystem();

        $uploads_dir            = \wp_get_upload_dir();
        $log_file               = $uploads_dir['basedir'] . '/shopwp.log';

        $removed_old_sync_log   = $Filesystem->delete_file($log_file);
        $bulk_query             = $this->DB_Settings_General->col('bulk_products_query', 'string');

        if (empty($bulk_query) || $bulk_query === '') {
            $bulk_query = '*';
        }

        $this->sync_shopify_data($bulk_query, [
            'is_cron' => true
        ]);

    }

    public function shopwp_bulk_recheck_cron_hook_stuff() {

        $current_bulk_operations = $this->Webhooks->admin_api_get_current_bulk_operation();

        if ($current_bulk_operations->status !== 'RUNNING' && $current_bulk_operations->status !== 'CANCELING') {

            // unschedule cron
            $this->Webhooks->unschedule_cron('shopwp_bulk_recheck_cron_hook');
            $bulk_query = $this->DB_Settings_General->col('bulk_products_query', 'string');

            if (empty($bulk_query) || $bulk_query === '') {
                $bulk_query = '*';
            }

            $this->sync_shopify_data($bulk_query, [
                'is_cron' => true
            ]);

        }
        
    }

    public function register_routes() {
        $this->api_route('/syncing/expire', 'POST', [$this, 'handle_expire_sync']);
        $this->api_route('/products/types', 'GET', [$this, 'handle_get_all_product_types']);
        $this->api_route('/products/vendors', 'GET', [$this, 'handle_get_all_product_vendors']);
        $this->api_route('/products/tags', 'GET', [$this, 'handle_get_all_product_tags']);
        $this->api_route('/products/collections', 'POST', [$this, 'handle_get_products_from_collections']);
        $this->api_route('/products', 'POST', [$this, 'handle_get_products']);
        $this->api_route('/syncing/data', 'POST', [$this, 'handle_sync_data']);
        $this->api_route('/syncing/remove', 'POST', [$this, 'handle_remove_synced_data']);
        $this->api_route('/syncing/remove/images', 'POST', [$this, 'handle_remove_synced_images']);
        $this->api_route('/posts', 'POST', [$this, 'handle_get_posts']);
    }

    public function init()
    {
        add_action('rest_api_init', [$this, 'register_routes']);
    }
}
