<?php

namespace ShopWP\API\Syncing;

use ShopWP\Messages;

if (!defined('ABSPATH')) {
    exit();
}

class Status extends \ShopWP\API
{
    public $DB_Settings_Syncing;

    public function __construct($DB_Settings_Syncing)
    {
        $this->DB_Settings_Syncing  = $DB_Settings_Syncing;
    }

    public function calc_syncing_total($totals) {
        return array_reduce($totals, function($carry, $current) {
            return $carry + $current;
        }, 0);
    }

    public function get_syncing_status($request)
    {

        $is_running                 = $request->get_param('isRunning');
        $current_tool               = $request->get_param('currentTool');
        $total_to_process           = $this->DB_Settings_Syncing->syncing_totals();
        $total_processed            = $this->DB_Settings_Syncing->syncing_current_amounts();
        $uploads_dir                = \wp_get_upload_dir();
        $log_full                   = file_get_contents($uploads_dir['basedir'] . '/shopwp.log', false, null);

        if (empty($log_full)) {
            $log = 'No syncing log found';
        } else {
            $log = substr($log_full, -10000);
        }

        $final_total_to_process     = $this->calc_syncing_total($total_to_process);
        $final_total_processed      = $this->calc_syncing_total($total_processed);

        $sync_data = [
            'isSyncing'                     => $this->DB_Settings_Syncing->is_syncing(),
            'isBusy'                        => $this->DB_Settings_Syncing->is_syncing(),
            'recentlySyncdMediaRef'         => $this->DB_Settings_Syncing->get_recently_syncd_media_ref(),
            'currentStepText'               => $this->DB_Settings_Syncing->get_current_syncing_step_text(),
            'totalToProcess'                => $final_total_to_process,
            'totalProcessed'                => $final_total_processed,
            'finishedDataDeletions'         => $this->get_syncing_status_removal(),
            'finishedWebhooksDeletions'     => $this->get_syncing_status_webhooks_deletion(),
            'finishedWebhooksConnection'    => $this->get_syncing_status_webhooks_connection(),
            'finishedRemovingConnection'    => $this->get_syncing_status_remove_connection(),
            'finishedMedia'                 => $this->get_syncing_status_media(),
            'finishedSyncingData'           => $this->get_has_finished_syncing_data(),
            'percentCompletedRemoval'       => $this->get_percent_completed_removal(),
            'percentCompleted'              => $this->DB_Settings_Syncing->calc_percent_completed($total_processed, $total_to_process),
            'syncingErrors'                 => maybe_unserialize($this->get_syncing_errors()),
            'syncingWarnings'               => maybe_unserialize($this->DB_Settings_Syncing->get_syncing_warnings()),
            'syncLog'                       => $log,
            'syncingTotals'                 => $total_to_process,
            'syncingCurrentAmounts'         => $total_processed,
            'nextCronTime'                  => \wp_next_scheduled('shopwp_auto_sync_cron_callback'),
            'isRunning'                     => $is_running
        ];

        return \wp_send_json_success($sync_data);

    }

    public function get_syncing_errors() {

        $any_errors = maybe_unserialize($this->DB_Settings_Syncing->get_column_single('syncing_errors'));

        if (empty($any_errors) || empty($any_errors[0]->syncing_errors)) {
            return false;
        }
                
        return maybe_unserialize($any_errors[0]->syncing_errors);

    }

    public function get_percent_completed_removal() {

        if (!$this->DB_Settings_Syncing->is_syncing()) {
            return 100;
        }

        $percent_completed_removal = $this->DB_Settings_Syncing->col('percent_completed_removal', 'int');

        return $percent_completed_removal;

    }

    public function get_has_finished_syncing_data() {
        
        $syncing_current_amounts_products = $this->DB_Settings_Syncing->col('syncing_current_amounts_products', 'int');
        $syncing_totals_products = $this->DB_Settings_Syncing->col('syncing_totals_products', 'int');

        if ($syncing_current_amounts_products === 0 && $syncing_totals_products === 0) {
            return false;
        }

        if ($syncing_current_amounts_products === $syncing_totals_products) {
            return true;
        }

        return false;
    }

    public function get_syncing_status_webhooks_deletion()
    {
        return $this->DB_Settings_Syncing->col('finished_webhooks_deletions', 'bool');
    }

    public function get_syncing_status_remove_connection()
    {
        return $this->DB_Settings_Syncing->col('finished_removing_connection', 'bool');
    }

    public function get_syncing_status_webhooks_connection()
    {
        
        $syncing_current_amounts_webhooks = $this->DB_Settings_Syncing->col('syncing_current_amounts_webhooks', 'int');
        $syncing_totals_webhooks = $this->DB_Settings_Syncing->col('syncing_totals_webhooks', 'int');

        if ($syncing_current_amounts_webhooks === 0 && $syncing_totals_webhooks === 0) {
            return false;
        }

        if ($syncing_current_amounts_webhooks === $syncing_totals_webhooks) {
            return true;
        }

        return false;
    }

    public function get_syncing_status_removal()
    {
        if (!$this->DB_Settings_Syncing->is_syncing()) {
            return true;
        }

        return $this->DB_Settings_Syncing->col('finished_data_deletions', 'bool');

    }

    public function get_syncing_status_media()
    {
        if (!$this->DB_Settings_Syncing->is_syncing()) {
            return true;
        }

        return $this->DB_Settings_Syncing->col('finished_media', 'bool');

    }

    public function register_route_syncing_status()
    {
        return register_rest_route(
            SHOPWP_SHOPIFY_API_NAMESPACE,
            '/syncing/status',
            [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => [$this, 'get_syncing_status'],
                    'permission_callback' => [$this, 'pre_process'],
                ],
            ]
        );
    }

    public function expire_sync()
    {

        $result = $this->handle_response([
            'response' => $this->DB_Settings_Syncing->expire_sync(),
        ]);

        update_option('shopwp_should_flush_rewrite_rules', 1);
        do_action('shopwp_processing_completed');
       
        return $result;
    }

    public function init()
    {
        add_action('rest_api_init', [$this, 'register_route_syncing_status']);
    }
}
