<?php

namespace ShopWP;

use ShopWP\Options;
use ShopWP\Utils;
use ShopWP\Utils\Data;

if (!defined('ABSPATH')) {
    exit();
}

class Data_Bridge
{
    
    public $plugin_settings;
    public $Render_Products_Defaults;
    public $Render_Cart_Defaults;
    public $Render_Collections_Defaults;
    public $Render_Search_Defaults;
    public $Render_Storefront_Defaults;
    public $Render_Reviews_Defaults;

    public function __construct($plugin_settings, $Render_Cart_Defaults, $Render_Collections_Defaults, $Render_Products_Defaults, $Render_Search_Defaults, $Render_Storefront_Defaults, $Render_Reviews_Defaults)
    {

        $this->plugin_settings              = $plugin_settings;
        $this->Render_Cart_Defaults         = $Render_Cart_Defaults;
        $this->Render_Collections_Defaults  = $Render_Collections_Defaults;
        $this->Render_Products_Defaults     = $Render_Products_Defaults;
        $this->Render_Search_Defaults       = $Render_Search_Defaults;
        $this->Render_Storefront_Defaults   = $Render_Storefront_Defaults;
        $this->Render_Reviews_Defaults      = $Render_Reviews_Defaults;

    }

    public function replace_lang_in_rest_url($found_lang_code, $rest_url) {

        if ( strlen(trim($found_lang_code)) == 0) {
            return $rest_url;
        }

        return str_replace("/" . $found_lang_code, "", $rest_url);
    }

    public function sanitize_rest_url_for_translate_press($rest_url, $TRP_LANGUAGE) {

        if ($TRP_LANGUAGE === 'en_US') {
            return $rest_url;
        }
        
        $trp = \TRP_Translate_Press::get_trp_instance();
        $url_converter = $trp->get_component('url_converter');
        $found_lang_code = $url_converter->get_url_slug($TRP_LANGUAGE);

        return $this->replace_lang_in_rest_url($found_lang_code, $rest_url);
    }

    public function sanitize_rest_url_for_wpml($rest_url) {

        $current_lang = apply_filters( 'wpml_current_language', NULL );

        return $this->replace_lang_in_rest_url($current_lang, $rest_url);
    }

    public function replace_rest_protocol()
    {
        $rest_url = \get_rest_url();

        global $TRP_LANGUAGE;

        if (!empty($TRP_LANGUAGE)) {
            $rest_url = $this->sanitize_rest_url_for_translate_press($rest_url, $TRP_LANGUAGE);

        } else if (defined('WPML_PLUGIN_BASENAME')) {
            $rest_url = $this->sanitize_rest_url_for_wpml($rest_url);
        }

        if (\is_ssl()) {
            return str_replace("http://", "https://", $rest_url);
        }

        return $rest_url;
    }

    public function starts_with($haystack, $needle) {
        $length = strlen( $needle );
        return substr( $haystack, 0, $length ) === $needle;
    }
    
    public function ends_with($haystack, $needle) {
        
        $length = strlen( $needle );
        
        if (!$length) {
        return true;
        }
    
        return substr( $haystack, -$length ) === $needle;
    
    }
    
    public function get_available_pages()
    {
        $pages = \get_pages();
    
        $final_pages = array_map(function ($page) {
            return [
                'id' => $page->ID,
                'post_title' => $page->post_title,
                'permalink' => get_permalink($page->ID)
            ];
        }, $pages);

        return $final_pages;
    }    

    public function get_has_connection($connection)
    {
        if (empty($connection)) {
            return false;
        }

        if (
            empty($connection['api_password']) ||
            empty($connection['domain'])
        ) {
            return false;
        }

        return true;
    }

    public function localhost_whitelist() {
        return [
            '127.0.0.1',
            '::1',
            '172.16.238.1',
            'localhost'
        ];
    }

    public function is_localhost() {
        if (in_array($_SERVER['REMOTE_ADDR'], $this->localhost_whitelist())) {
            return true;
        }

        return false;
    }

    public function get_site_lang() {

        $lang = \get_locale();

        if ( strlen( $lang ) > 0 ) {

            if (Utils::str_contains($lang, '_')) {
                $lang = explode( '_', $lang )[0];

                return $lang;

            } else {

                return $lang;
            }
            
        } else {
            return 'en';
        }

    }

    public function get_settings($is_admin)
    {
        global $post;
        global $wp_version;

        if ($wp_version === '6.2-RC4') {
            $wp_ver = '6.2';
            
        } else {
            $wp_ver = $wp_version;
        }

        $active_metafields = \maybe_unserialize(Options::get('shopwp_active_metafields'));

        if (empty($active_metafields)) {
            $active_metafields = false;
        }

        $general = Data::sanitize_settings($this->plugin_settings['general']);

        if ($general['enable_data_cache']) {
            $cache_length = $general['data_cache_length'];

        } else {
            $cache_length = false;
        }

        // Don't bring certain fields into the front-end ...
        if (!is_admin()) {
            $general['sync_by_collections'] = false;
            $current_locale = \apply_filters('shopwp_buyer_identity_language', $general['language_code'], $general['language_code']);

        } else {
            $current_locale = $this->get_site_lang();
        }

        $upload_dir_url     = \wp_get_upload_dir();
        $translations       = $this->update_translations($current_locale);

        if (!is_admin()) {
            unset($translations['a']);
        }

        $settings = [
            'cart'          => $this->Render_Cart_Defaults->all_attrs([], $translations),
            'collections'   => $this->Render_Collections_Defaults->all_attrs([], $translations),
            'products'      => $this->Render_Products_Defaults->all_attrs([], $translations),
            'search'        => $this->Render_Search_Defaults->all_attrs([], $translations),
            'storefront'    => $this->Render_Storefront_Defaults->all_attrs([], $translations),
            'reviews'       => $this->Render_Reviews_Defaults->all_attrs([], $translations),
            'syncing' => [
                'reconnectingWebhooks'          => false,
                'hasConnection'                 => $this->get_has_connection($this->plugin_settings['connection']),
                'syncLogUrl'                    => is_admin() ? $upload_dir_url['baseurl'] . '/shopwp.log' : false,
                'isSyncing'                     => $this->plugin_settings['syncing']['is_syncing'],
                'manuallyCanceled'              => false,
                'isClearing'                    => false,
                'isDisconnecting'               => false,
                'isConnecting'                  => false,
                'isAutomaticSyncingConnected'   => $this->plugin_settings['syncing']['is_automatic_syncing_connected'],
                'isRemovingImages'              => $this->plugin_settings['syncing']['is_removing_images']
            ],
            'general' => $general,
            'connection' => [
                'masked' => Data::mask_api_values($this->plugin_settings['connection']),
                'storefront' => [
                    'domain' =>
                        $this->plugin_settings['connection']['domain'],
                    'storefrontAccessToken' =>
                        $this->plugin_settings['connection'][
                            'storefront_access_token'
                        ],
                ]
            ],
            'notices' => $this->plugin_settings['notices'],
            'api' => [
                'namespace' => SHOPWP_SHOPIFY_API_NAMESPACE,
                'restUrl' => $this->replace_rest_protocol(),
                'nonce' => \wp_create_nonce('wp_rest'),
            ],
            'misc' => [
                'availablePages' => $is_admin ? $this->get_available_pages() : false,
                'postID' => $post ? $post->ID : false,
                'postType' => $post ? $post->post_type : false,
                'postTitle' => $post ? $post->post_title : false,
                'postURL' => $is_admin && $post ? get_permalink($post->ID) : false,
                'isMobile' => \wp_is_mobile(),
                'pluginsDirURL' => \plugin_dir_url(dirname(__FILE__)),
                'pluginsDistURL' => plugin_dir_url(dirname(__FILE__)) . 'dist/',
                'adminURL' => \get_admin_url(),
                'siteUrl' => \site_url(),
                'isSingularProducts' => \is_singular('wps_products'),
                'isSingularCollections' => \is_singular('wps_collections'),
                'siteDomain' => parse_url(site_url())['host'],
                'isAdmin' => $is_admin,
                'isReact18' => version_compare($wp_ver, '6.2', '>='),
                'latestVersion' => SHOPWP_NEW_PLUGIN_VERSION,
                'isLocalhost' => $this->is_localhost(),
                'siteLang' => $this->get_site_lang(),
                'hasTranslator' => defined('SHOPWP_DOWNLOAD_ID_TRANSLATOR_EXTENSION') ? true : false,
                'hasRecharge' => defined('SHOPWP_DOWNLOAD_ID_RECHARGE_EXTENSION') ? true : false,
                'hasElementor' => defined('SHOPWP_DOWNLOAD_ID_ELEMENTOR_EXTENSION') ? true : false,
                'hasYotpo' => defined('SHOPWP_DOWNLOAD_ID_YOTPO_REVIEWS_EXTENSION') ? true : false,
                'timers' => [
                    'cronNextSync' => \wp_next_scheduled('shopwp_auto_sync_cron_callback')
                ],
                'cache_length' => $cache_length,
                'theme' => 'light'
            ],
            'collectionsData' => $is_admin ? maybe_unserialize(Transients::get('shopwp_all_collections')) : false,
            'metafields' => $is_admin ? $active_metafields : false,
            't' => $translations
        ]; 

        if ($is_admin) {
            $settings['misc']['isSSL'] = \is_ssl();
        }

        return apply_filters('shopwp_settings', $settings);

    }

    public function update_translations($locale) {

        global $l10n;

        $path = SHOPWP_PLUGIN_DIR . SHOPWP_LANGUAGES_FOLDER . '/';

        $final_path = $path . 'shopwp' . '-' . strtolower($locale) . '.mo';
        $final_path_with_ending = $path . 'shopwp' . '-' . $locale . '_' . strtoupper($locale) . '.mo';

        $unload = \unload_textdomain('shopwp');
        $load = \load_textdomain('shopwp', $final_path);

        if (empty($load)) {
            $load_again = \load_textdomain('shopwp', $final_path_with_ending);
        }

        $all_string_cats = $this->translations();

        return $all_string_cats;
    }

    public function translations() {
        return [
            'e' => [
                'sslBad'                            => __("ShopWP error: Uh oh, it looks like your WordPress site does not have a valid SSL certificate installed. ShopWP will only work with a valid HTTPS connection. Please double check your SSL certificate and try again.", 'shopwp'),
                'noConnection'                      => __('No Shopify connection found. Please try reconnecting your Shopify store to ShopWP.', 'shopwp'),
                'noQueryParams'                     => __('ShopWP error: No query parameters were passed. Please clear your browser cache and try again.', 'shopwp'),
                'noClientCreds'                     => __('ShopWP error: Missing credentials Client object', 'shopwp'),
                'unknown'                           => __('ShopWP error: An unknown error occurred. Please clear the plugin cache and try again.', 'shopwp'),
                'badCreds'                          => __('ShopWP error: It looks like your Shopify credentials are incorrect. Please reconnect your Shopify store to the plugin and try again.', 'shopwp'),
                'badApiField'                       => __('ShopWP error: It looks like you may be trying to access a field on the Storefront API that does not exist.', 'shopwp'),
                'invalidLineitemsType'              => __('ShopWP error: It looks like an invalid lineitems data type was found. Please clear your cache and try again.', 'shopwp'),
                'badNetwork'                        => __('ShopWP error: It looks like a network error occurred. Please make sure you\'re connected to the internet and that you\'re using a valid SSL certificate.', 'shopwp'),
                'rCurly'                            => __('ShopWP error: It looks like a syntax error occurred "}" (RCURLY) at [1, 10]. Please send ShopWP this message to fix.', 'shopwp'),
                'badProductIds'                     => __('ShopWP error: It appears that invalid product ids were used. Please clear your browser cache and reload the page.', 'shopwp'),
                'noCountryFound'                    => __('ShopWP Error: No country found by ISO code', 'shopwp'),
                'checkoutFailed'                    => __('Unable to checkout. Please reload the page and try again.', 'shopwp'),
                'requireSub'                        => __("This product cannot be purchased without a subscription. Please select a subscription and try again. You may need to turn the subscriptions feature on.", 'shopwp')
            ],
            'w' => [
                'noComponentData'                   => __('ShopWP warning: No component data found! Please double check your shortcode.', 'shopwp'),
                'noRoot'                            => __('ShopWP warning: No root shop container found. Made one on the fly.', 'shopwp'),
                'noBodyClass'                       => __('ShopWP warning: No <?php body_class();?> found in theme. Added shopwp to body on the fly.', 'shopwp'),
                'maxCartTotal'                      => __('Sorry, but you\'ve reached the maximum total allowed for your cart.', 'shopwp'),
                'noSubsFound'                       => __('No subscriptions found', 'shopwp'),
                'noProdFound'                       => __("ShopWP warning: No product found. You must pass a valid product id to show the create review form.", 'shopwp'),
                'variantUnavailable'                => __("Sorry, this variant is not available. Please try a different combination.", 'shopwp'),
            ],
            'n' => [
                'noItemsLeft'                       => __('No items left to show', 'shopwp'),
                'noResultsFound'                    => __('No results found', 'shopwp'),
                'noCollectionsLeft'                 => __('No collections left to show', 'shopwp'),
                'leftInStock'                       => __('Hurry, we\'re almost out of this one!', 'shopwp'),
                'limitedTotal'                      => __('Note: the quantity you\'re trying to purchase may be limited at checkout.', 'shopwp'),
                'totalTax'                          => __('The total tax may change on the final checkout page. This is not a final estimate.', 'shopwp'),
                'noDiscountFound'                   => __('No discount found! Please enter one below.', 'shopwp'),
                'addedDiscount'                     => __('Successfully added discount code', 'shopwp'),
                'removedDiscount'                   => __('Successfully removed discount code', 'shopwp'),
                'discountApplied'                   => __('Discount applied', 'shopwp'),
                'reviewSubmitThanks'                => __('Thanks! Your review has been submitted successfully.', 'shopwp'),
                'reviewEnterEmail'                  => __("You must enter your email", 'shopwp'),
                'reviewEnterTitle'                  => __("You must give your review a title", 'shopwp'),
                'reviewEnterReview'                 => __("This is the most important part! You must enter a review.", 'shopwp'),
                'reviewEnterName'                   => __("You must enter your name", 'shopwp'),
                'reviewPlaceTitle'                  => __("Give your review a title", 'shopwp'),
                'reviewPlaceWrite'                  => __("Write your review here", 'shopwp'),
                'bodyOfReview'                      => __("Body of Review", 'shopwp'),
                'reviewTitle'                       => __("Review Title", 'shopwp'),
                'reviewRating'                      => __("Rating", 'shopwp'),
                'custOrdersSync'                    => __("Customers and Orders may only be synced individually by themselves. You must first deselect everything else above.", 'shopwp'),
                'subDetailsContent'                 => __('Products are automatically delivered on your schedule. No obligation, modify or cancel your subscription anytime.', 'shopwp'),
                'outOfStock'                        => __('Out of stock. Please try selecting a different variant combination.', 'shopwp'),
            ],
            'l' => [
                'terms'                             => __('I agree with the terms and conditions.', 'shopwp'),
                'collectionDefault'                 => __('Collection Default', 'shopwp'),
                'recentlyUpdated'                   => __('Recently Updated', 'shopwp'),
                'productVendor'                     => __('Product Vendor', 'shopwp'),
                'recentlyAdded'                     => __('Recently Added', 'shopwp'),
                'titleDes'                          => __('Title (A-Z)', 'shopwp'),
                'titleAsc'                          => __('Title (Z-A)', 'shopwp'),
                'priceLowToHigh'                    => __('Price (low to high)', 'shopwp'),
                'priceHighToLow'                    => __('Price (high to low)', 'shopwp'),
                'seeMore'                           => __('See more', 'shopwp'),
                'customerReviews'                   => __('Customer Reviews', 'shopwp'),
                'searchStore'                       => __('Search the store', 'shopwp'),
                'soldOut'                           => __('Sold out', 'shopwp'),
                'loadMore'                          => __('Load more', 'shopwp'),
                "outOfStock"                        => __('Out of stock', 'shopwp'),
                'clearSelections'                   => __('Clear selections', 'shopwp'),
                'added'                             => __('Added', 'shopwp'),
                'quantity'                          => __('Quantity', 'shopwp'),
                'noResultsText'                     => __('No items left to show', 'shopwp'),
                'remove'                            => __('Remove', 'shopwp'), 
                'yourCartEmpty'                     => __('Your cart is empty', 'shopwp'),
                'checkoutNotes'                     => __('Checkout notes', 'shopwp'),
                'cartTitle'                         => __('Shopping cart', 'shopwp'),
                'updatingCart'                      => __('Updating cart', 'shopwp'),
                'subDetails'                        => __('Subscription details', 'shopwp'),
                'subDetailsHeading'                 => __('How subscriptions work:', 'shopwp'),
                "products"                          => __('Products', 'shopwp'),
                "product"                           => __('Single Product', 'shopwp'),
                "buyButton"                         => __('Buy Button', 'shopwp'),
                "subscriptions"                     => __('Subscriptions', 'shopwp'),
                "title"                             => __('Title', 'shopwp'),
                "titleColor"                        => __('Title Color', 'shopwp'),
                "vendor"                            => __('Vendor', 'shopwp'),
                "tag"                               => __('Tag', 'shopwp'),
                "general"                           => __('General', 'shopwp'),
                "cart"                              => __('Cart', 'shopwp'),
                "colors"                            => __('Colors', 'shopwp'),
                "pagination"                        => __('Pagination', 'shopwp'),
                "pricing"                           => __('Pricing', 'shopwp'),
                "country"                           => __('Country', 'shopwp'),
                "price"                             => __('Price', 'shopwp'),
                "layout"                            => __('Layout', 'shopwp'),
                "search"                            => __('Search', 'shopwp'),
                "images"                            => __('Images', 'shopwp'),
                "checkout"                          => __('Checkout', 'shopwp'),
                "description"                       => __('Description', 'shopwp'),
                "inStock"                           => __('In stock', 'shopwp'),
                'carousel'                          => __('Carousel?', 'shopwp'),
                'subscription'                      => __('Subscription', 'shopwp'),
                'loading'                           => __('Loading', 'shopwp'),
                'sort'                              => __('Sort by', 'shopwp'),
                'prices'                            => __('Prices', 'shopwp'),
                'sorting'                           => __('Sorting', 'shopwp'),
                'oneTimePurchase'                   => __('One-time purchase', 'shopwp'),
                'save'                              => __('Save', 'shopwp'),
                'defaultTitle'                      => __('Default Title', 'shopwp'),
                'estTax'                            => __('Estimated tax:', 'shopwp'),
                'showCarouselDots'                  => __('Show carousel dots?', 'shopwp'),
                'scrollInfinite'                    => __('Scroll infinitely?', 'shopwp'),
                'filtering'                         => __('Filtering', 'shopwp'),
                'fullWidth'                         => __('Full width', 'shopwp'),
                'connect'                           => __('Connect', 'shopwp'),
                'sync'                              => __('Sync', 'shopwp'),
                'settings'                          => __('Settings', 'shopwp'),
                'license'                           => __('License', 'shopwp'),
                'extensions'                        => __('Extensions', 'shopwp'),
                'docs'                              => __('Docs', 'shopwp'),
                'visualBuilder'                     => __('Visual Builder', 'shopwp'),
                'viewThemHere'                      => __('View them here', 'shopwp'),
                'disconnect'                        => __('Disconnect your Shopify store', 'shopwp'),
                'connectStore'                      => __('Connect your Shopify store', 'shopwp'),
                'connected'                         => __('Connected', 'shopwp'),
                'disconnected'                      => __('Disconnected', 'shopwp'),
                'usefulLinks'                       => __('Useful Plugin Links', 'shopwp'),
                'name'                              => __('Name', 'shopwp'),
                'email'                             => __('Email', 'shopwp'),
                'acLimit'                           => __('Activation limit', 'shopwp'),
                'expires'                           => __('Expires', 'shopwp'),
                'unlimited'                         => __('Unlimited', 'shopwp'),
                'neverExp'                          => __('Never expires', 'shopwp'),
                'cartNotesPH'                       => __('Cart notes placeholder', 'shopwp'),
                'cartNotesPHContent'                => __('Is this a gift? Enter your special message here. (Limit 100 characters)', 'shopwp'),
                'cartTerms'                         => __('Cart terms', 'shopwp'),
                'domainInclude'                     => __(' Please do not include "https://" or "www".', 'shopwp'),
                'addToCart'                         => __('Add to cart', 'shopwp'),
                'viewProduct'                       => __('View product', 'shopwp'),
                'off'                               => __('off', 'shopwp'),
                'delete'                            => __('Delete', 'shopwp'),
                'closeCart'                         => __('Close Cart', 'shopwp'),
                'learnMore'                         => __('Learn more', 'shopwp'),
                'currency'                          => __('Currency', 'shopwp'),
                'lang'                              => __('Language', 'shopwp'),
                'discountCode'                      => __('Discount code', 'shopwp'),
                'apply'                             => __('Apply', 'shopwp'),
                'selectDelivery'                    => __('Delivery', 'shopwp'),
                'selectA'                           => __('Please select a ', 'shopwp'),
                'sale'                              => __('Sale!', 'shopwp'),
                'submitReview'                      => __('Submit Review', 'shopwp'),
                'writeAReview'                      => __('Write a review', 'shopwp'),
                'noReviews'                         => __('No reviews yet for this product.', 'shopwp'),
                'outOfStockNotify'                  => __('Out of stock (Notify me when it\'s available', 'shopwp'),
                'selectLang'                        => __('Select language ...', 'shopwp'),
                'verifiedBuyer'                     => __('Verified buyer', 'shopwp'),
                'loadReviews'                       => __('Load more reviews', 'shopwp'),
                'addApiKeys'                        => __('Please add your Yotpo API keys to show reviews', 'shopwp'),
                'subtotal'                          => __('Subtotal:', 'shopwp'),
                'beginCheckout'                     => __('Begin checkout', 'shopwp'),
                'filterBy'                          => __('Filter by:', 'shopwp'),
                'pageSize'                          => __('Page size', 'shopwp'),
                'clearAll'                          => __('Clear all', 'shopwp'),
                'tags'                              => __('Tags', 'shopwp'),
                'types'                             => __('Types', 'shopwp'),
                'vendors'                           => __('Vendors', 'shopwp'),
                'bestSelling'                       => __('Best Selling', 'shopwp'),
                'collectionManual'                  => __('Manual', 'shopwp'),
                'productType'                       => __('Product Type', 'shopwp'),
                'createdAt'                         => __('Created At', 'shopwp'),
                'updatedAt'                         => __('Updated At', 'shopwp'),
                "availSale"                         => __('Available for sale', 'shopwp'),
                'collections'                       => __('Collections', 'shopwp'),
                '_10'                               => __('10', 'shopwp'),
                '_25'                               => __('25', 'shopwp'),
                '_50'                               => __('50', 'shopwp'),
                '_100'                              => __('100', 'shopwp'),
                'priceLvl1'                         => __("$0.00 - $15.00", "shopwp"),
                'priceLvl2'                         => __("$15.00 - $25.00", "shopwp"),
                'priceLvl3'                         => __("$25.00 - $50.00", "shopwp"),
                'priceLvl4'                         => __("$50.00 - $100.00", "shopwp"),
                'priceLvl5'                         => __("$100.00 +", "shopwp"),
                'review'                            => __('Review', 'shopwp'),
                'reviews'                           => __('Reviews', 'shopwp'),
            ],
            // Admin only
            'a' => [
                'requireNotes'                      => __('Require notes?', 'shopwp'),
                'dlSyncLog'                         => __('Download syncing log', 'shopwp'),
                'syncInProg'                        => __('Syncing in progress ...', 'shopwp'),
                'noSyncInProg'                      => __('No sync in progress', 'shopwp'),
                'upsellHeading'                     => __('Upsell heading', 'shopwp'),
                'upsellDescription'                 => __('Upsell description', 'shopwp'),
                'upsellApplyDiscount'               => __('Apply discount?', 'shopwp'),
                'upsellApplyDiscountDesc'           => __('Allows you to add a discount when an upsell is purchased', 'shopwp'),
                'selectOrSearch'                    => __('Select or search for a product', 'shopwp'),
                'password'                          => __('Password:', 'shopwp'),
                'emailOr'                           => __('Email or username', 'shopwp'),
                'selectMore'                        => __('Command or Control click to select more than one', 'shopwp'),
                'selectWebhooks'                    => __('Select the webhooks to sync:', 'shopwp'),
                'afterWebhookSave'                  => __('After selecting your webhooks, save the ShopWP settings and go to the "Sync" tab', 'shopwp'),
                'requiresThe'                       => __('This feature requires the ', 'shopwp'),
                'allowAltWebhook'                   => __('Use alternative webhook authentication', 'shopwp'),
                'currencyDisplayStyle'              => __('Currency display style', 'shopwp'),
                'currencySymbol'                    => __('Symbol (example: $19.99)', 'shopwp'),
                'currencyCode'                      => __('Code (example: USD 19.99)', 'shopwp'),
                'currencyName'                      => __('Name (example: 19.99 US dollars)', 'shopwp'),
                'hideDecimals'                      => __('Hide decimals?', 'shopwp'),
                'showPriceRange'                    => __('Show price range', 'shopwp'),
                'showPriceRangeHelp'                => __('When enabled, shows a product\'s range of variant prices. E.g., \'From $1.00 - $5.00\' instead of just \'$1.00\'.', 'shopwp'),
                'colors'                            => __('Colors', 'shopwp'),
                'data'                              => __('Data', 'shopwp'),
                'addToCartBtns'                     => __('Add to cart buttons', 'shopwp'),
                'showDefaultHeading'                => __('Show Default Page Heading?', 'shopwp'),
                'showDefaultHeadingHelp'            => __('Hide / show the heading found on the default Products page.', 'shopwp'),
                'showImageZoom'                     => __('Show image zoom', 'shopwp'),
                'customFeatImgSizing'               => __('Custom featured image sizing', 'shopwp'),
                'includeMeta'                       => __('Include metafields on product data?', 'shopwp'),
                'showProdDescOnDefault'             => __('Show product descriptions on default page', 'shopwp'),
                'showProdDescOnDefaultHelp'         => __('Determines whether product descriptions will show on the default product pages.', 'shopwp'),
                'customThumbImgSizing'              => __('Custom thumbnail image sizing', 'shopwp'),
                'showReviews'                       => __('Show reviews on PDP', 'shopwp'),
                'showReviewsHelp'                   => __('When this is enabled, ShopWP will show Yotpo reviews on the product detail page (PDP) below the product information.', 'shopwp'),
                'searchExact'                       => __('Search exact match?', 'shopwp'),
                'searchExactHelp'                   => __('Determines whether to match exactly what the user types when searching product titles. For example, if the user types \'Shoe\', only products with the title \'Shoe\' will be returned.', 'shopwp'),
                'showSubscriptions'                 => __('Show subscriptions by default', 'shopwp'),
                'showSubscriptionsHelp'             => __('Will show the subscriptions widget on every product.', 'shopwp'),
                'autoSyncInterval'                  => __('Auto sync cron interval', 'shopwp'),
                'autoSyncIntervalHelp'              => __('How often auto syncing should sync in seconds. (43200 = 12 hours)', 'shopwp'),
                'productsSyncQuery'                 => __('Products sync query', 'shopwp'),
                'productsSyncQueryNotUsed'          => __('This sync query won\'t be used because you\'re syncing from collections.', 'shopwp'),
                'productsSyncQueryTwo'              => __('Syncing query', 'shopwp'),
                'productsSyncQueryHelp'             => __('Using * will sync all products', 'shopwp'),
                'productImages'                     => __('Product images', 'shopwp'),
                'productOptions'                    => __('Product options', 'shopwp'),
                'orders'                            => __('Orders', 'shopwp'),
                'productCommon'                     => __('Product common (title, description, pricing, etc)', 'shopwp'),
                'productTags'                       => __('Product tags', 'shopwp'),
                'productVariants'                   => __('Product variants', 'shopwp'),
                'syncProductsFromCollections'       => __('Sync products from collections', 'shopwp'),
                'loadingCollections'                => __('Loading collections to sync from ... ', 'shopwp'),
                'reconnect'                         => __('Reconnect', 'shopwp'),
                'deleteOldData'                     => __('Delete old data before each sync?', 'shopwp'),
                'syncProductAndCollectionPages'     => __('Sync product and collection detail pages?', 'shopwp'),
                'syncProductAndCollectionPagesNote' => __('Creating product detail pages requires that you manually sync your products from within the plugin\'s Sync tab. Make sure to do this after you\'re done here.', 'shopwp'),
                'syncFeatImgOnly'                   => __('Sync featured images only?', 'shopwp'),
                'syncIntoPostMeta'                  => __('Sync data into post meta fields?', 'shopwp'),
                'syncFeatImage'                     => __('Sync featured images?', 'shopwp'),
                'syncIntoDraft'                     => __('Sync products into draft mode?', 'shopwp'),
                'skipDraftProd'                     => __('Skip draft products during sync?', 'shopwp'),
                'itemsProcessedPerChunk'            => __('Items processed per chunk', 'shopwp'),
                'syncingUrl'                        => __('Syncing URL', 'shopwp'),
                'syncingUrlHelp'                    => __('This URL must match your WordPress domain', 'shopwp'),
                'localhostWarning'                  => __('Hey there! It looks like you\'re using localhost. In order for the syncing to work your site must be publicly accessible. Please make sure that you\'ve set the above Syncing URL to a publicly accessible URL using something like ngrok.', 'shopwp'),
                'learnHowToSetup'                   => __(' Learn how to set this up', 'shopwp'),
                'youStillNeed'                      => __('Warning: You still need to ', 'shopwp'),
                'connectStore'                      => __('connect your Shopify store', 'shopwp'),
                'oneTimeSync'                       => __('One-time Sync', 'shopwp'),
                'oneTimeSyncHelp'                   => __('If you\'re noticing various changes not appearing, try clearing the ShopWP transient cache here.', 'shopwp'),
                'syncShopifyData'                   => __('Sync Shopify Data', 'shopwp'),
                'autoSync'                          => __('Sync Shopify Data', 'shopwp'),
                'autoSyncingHelp'                   => __('Enables automatic post syncing. When you add or remove products inside Shopify, ShopWP will automatically add or remove product posts based on the action performed.', 'shopwp'),
                'connectAutoSync'                   => __('Connect Auto Sync', 'shopwp'),
                'syncingPleaseWait'                 => __('Syncing, please wait ...', 'shopwp'),
                'removeAllData'                     => __('Remove all synced data', 'shopwp'),
                'removeAllDataHelp'                 => __('This tool will remove all previously synced product and collection detail pages. Useful for clearing any corrupt or stale data.', 'shopwp'),
                'removeAllSyncedImgs'               => __('Remove all synced images', 'shopwp'),
                'removeAllSyncedImgsHelp'           => __('This tool will remove all previously synced product and collection images from the WordPress media library.', 'shopwp'),
                'clearCache'                        => __('Clear cache', 'shopwp'),
                'clearCacheHelp'                    => __('If you\'re noticing various changes not appearing, try clearing the ShopWP transient cache here.', 'shopwp'),
                'willRemoveData'                    => __('Warning: This will remove all synced Shopify data from WordPress, including product and collection detail pages. Do you wish to continue?', 'shopwp'),
                'finishedSync'                      => __('Successfully finished syncing', 'shopwp'),
                'autoSyncCanceled'                  => __('Auto sync canceled successfully', 'shopwp'),
                'syncCanceledSuccess'               => __('Syncing canceled successfully', 'shopwp'),
                'syncCanceledSuccess'               => __('Syncing canceled successfully', 'shopwp'),
                'removedAllDataSuccess'             => __('Successfully removed all synced data', 'shopwp'),
                'clearedCacheSuccess'               => __('Successfully cleared cache', 'shopwp'),
                'removedImagesSuccess'              => __('Successfully removed synced images', 'shopwp'),
                'finishedWithErrors'                => __('Syncing finished with errors:', 'shopwp'),
                'zeroComplete'                      => __("0% complete, please wait ...", 'shopwp'),
                'waitingToComplete'                 => __("Waiting for sync please wait ...", 'shopwp'),
                'syncingComplete'                   => __("Syncing complete", 'shopwp'),
                'percComplete'                      => __("% complete, please wait ...", 'shopwp'),
                'loadingStore'                      => __("Loading store ...", 'shopwp'),
                'welcomeWizard'                     => __("Welcome! Let's connect your Shopify store.", 'shopwp'),
                'found'                             => __("🙌 Found, ", 'shopwp'),
                'settingDefaults'                   => __("Now we'll set some general plugin settings.", 'shopwp'),
                'configProdSettings'                => __("You're on a roll! Let's configure your products.", 'shopwp'),
                'almostDone'                        => __("Ok almost done, just a few more settings.", 'shopwp'),
                'niceWork'                          => __("👏 Nice work, you're connected!", 'shopwp'),
                'justDefaults'                      => __("These are just defaults. You can change them later.", 'shopwp'),
                'weAreLoading'                      => __("We are loading your Shopify store, one moment ...", 'shopwp'),
                'whereDoIFind'                      => __("Where do I find this?", 'shopwp'),
                'letsFinish'                        => __("Now, let's finish by adjusting some default plugin settings.", 'shopwp'),
                'letsFinishByEnteringDomain'        => __("Let's start by entering your Shopify domain below.", 'shopwp'),
                'goToSettings'                      => __("Go to the plugin settings", 'shopwp'),
                'viewYourProducts'                  => __("View your products:", 'shopwp'),
                'pdpReminder'                       => __("If you want product detail pages, remember to manually sync them by using the ", 'shopwp'),
                'pdpReminder2'                      => __(" Sync Product & Collection Detail Pages", 'shopwp'),
                'pdpReminder3'                      => __(" tool.", 'shopwp'),
                'backToWP'                          => __("Back to WordPress Dashboard", 'shopwp'),
                'errorAuthReq'                      => __("Uh oh, an error occurred while attempting to check for a valid authentication request. Please clear your browser cache, go to the plugin settings under the \"Connect\" tab, and try again.", 'shopwp'),
                'errorFetchExistingConnection'      => __("Uh oh, an error occurred while attempting to fetch an existing connection. Go to the plugin settings and verify that you have an active connection under the \"Connect\" tab.", 'shopwp'),
                'errorConnection'                   => __("Sorry there was an error during the connection. Please try again.", 'shopwp'),
                'domainError'                       => __("Your domain must end with .myshopify.com. Please do not include \"https://\" or \"www\".", 'shopwp'),
                'domainInvalid'                     => __("Your domain is invalid. Please make sure that it does not start with \"https://\" or \"www\". It should look like this: store.myshopify.com", 'shopwp'),
                'returnToPluginSettings'            => __("Return to plugin settings", 'shopwp'),
                'connectingToShopify'               => __("Connecting to Shopify ...", 'shopwp'),
                'finishingSetup'                    => __("Finishing Setup ...", 'shopwp'),
                'connectStore'                      => __("Connect your Shopify store", 'shopwp'),
                'continueToNextStep'                => __("Continue to Next Step", 'shopwp'),
                'saveAndFinish'                     => __("Save and Finish the Setup", 'shopwp'),
                'nextStep'                          => __("Continue to Next Step", 'shopwp'),
                'goBackToWP'                        => __("Go back to WordPress Dashboard", 'shopwp'),
                'prevStep'                          => __("Previous Step", 'shopwp'),
                'skipSetup'                         => __("Or skip the setup process", 'shopwp'),
                'findDomain'                        => __("Finding your Shopify domain", 'shopwp'),
                'connection'                        => __("Connection", 'shopwp'),
                'genSett'                           => __("General Settings", 'shopwp'),
                'prodSett'                          => __("Product Settings", 'shopwp'),
                'cartSett'                          => __("Cart Settings", 'shopwp'),
                'numOfSlides'                       => __("Number of slides to scroll", 'shopwp'),
                'numOfSlidesToShow'                 => __("Number of slides to show", 'shopwp'),
                'carouselSpeed'                     => __("Carousel speed", 'shopwp'),
                'filterByCollection'                => __("Filter by Collection", 'shopwp'),
                'filterByCollectionHelp'            => __("Filter products which belong to a collection. Only matches one collection.", 'shopwp'),
                'matchCriteria'                     => __("Match criteria", 'shopwp'),
                'matchCriteriaHelp'                 => __("Determines how a match is found when filtering products. \"OR\" will attempt to match any products from the set filters. \"AND\" will only match products when all filters are true. Default is AND", 'shopwp'),
                'and'                               => __("AND", 'shopwp'),
                'or'                                => __("OR", 'shopwp'),
                'createdAt'                         => __("Created At", 'shopwp'),
                'createdAtHelp'                     => __("Match product created at", 'shopwp'),
                'descColor'                         => __("Description Color:", 'shopwp'),
                'limitDesc'                         => __("Limit Description Length", 'shopwp'),
                'limitDescHelp'                     => __("Limits the number of characters", 'shopwp'),
                'enableDirectCheckout'              => __("Enable direct checkout", 'shopwp'),
                'excludeLayout'                     => __("Exclude from layout: ", 'shopwp'),
                'spaceBetween'                      => __("Space between items", 'shopwp'),
                'hideQuant'                         => __("hide quantity?", 'shopwp'),
                'showNextImgHover'                  => __("Show next image on hover", 'shopwp'),
                'enableFeatImgSizing'               => __("Enable featured image sizing", 'shopwp'),
                'infiniteScroll'                    => __("Infinite scroll?", 'shopwp'),
                'infiniteScrollOffset'              => __("Infinite scroll offset", 'shopwp'),
                'infiniteScrollOffsetHelp'          => __("Determines when infinite scroll begins", 'shopwp'),
                'itemsPerRow'                       => __("Items per row", 'shopwp'),
                'limitProds'                        => __("Limit products?", 'shopwp'),
                'limitProdsHelp'                    => __("Sets the number of products shown. This will take precedence over the page size setting.", 'shopwp'),
                'maxQuant'                          => __("Max quantity", 'shopwp'),
                'minQuant'                          => __("Min quantity", 'shopwp'),
                'shopifyDomain'                     => __("Shopify Domain", 'shopwp'),
                'noResultsText'                     => __("No results text", 'shopwp'),
                'pageSize'                          => __("Page size", 'shopwp'),
                'showPagination'                    => __("Show pagination", 'shopwp'),
                'productID'                         => __("Product ID", 'shopwp'),
                'productIDHelp'                     => __("Match product ids", 'shopwp'),
                'filterByProdType'                  => __("Filter by Product Type", 'shopwp'),
                'filterByProdTypeHelp'              => __("Match product types. Separate multiple by comma.", 'shopwp'),
                'linksShouldOpenIn'                 => __("Product links should open in:", 'shopwp'),
                'shouldLinkTo'                      => __("Products should link to", 'shopwp'),
                'shouldLinkToHelp'                  => __("You must sync your single pages before this link option will work.", 'shopwp'),
                'linkBuyButton'                     => __("Link while showing buy button", 'shopwp'),
                'reverseOrder'                      => __("Reverse order?", 'shopwp'),
                'showCompareAt'                     => __("Show compare at", 'shopwp'),
                'showCompareAtHelp'                 => __("Useful for showing sale pricing", 'shopwp'),
                'showFeatOnly'                      => __("Show featured only", 'shopwp'),
                'showImgCarousel'                   => __("Show images carousel", 'shopwp'),
                'showPriceRange'                    => __("Show price range", 'shopwp'),
                'showPriceUnder'                    => __("Show price next to variant button", 'shopwp'),
                'showZoom'                          => __("Show zoom", 'shopwp'),
                'showZoomHelp'                      => __("Will only work if not linking product to WordPress or Shopify", 'shopwp'),
                'enterAccessToken'                  => __("Enter your Storefront Access Token", 'shopwp'),
                'accessToken'                       => __("Storefront Access Token", 'shopwp'),
                'selectSubOnLoad'                   => __("Select subscription on load", 'shopwp'),
                'showSubsWidget'                    => __("Show subscriptions widget", 'shopwp'),
                'filterByTag'                       => __("Filter by Tag", 'shopwp'),
                'filterByTagHelp'                   => __("Match product tags. Separate multiple by comma.", 'shopwp'),
                'enableThumbSizing'                 => __("Enable thumbnail image sizing", 'shopwp'),
                'filterByTitle'                     => __("Filter by Title", 'shopwp'),
                'filterByTitleHelp'                 => __("Match product titles. Separate multiple by comma.", 'shopwp'),
                'updatedAt'                         => __("Updated At", 'shopwp'),
                'updatedAtHelp'                     => __("Match product updated at", 'shopwp'),
                'filterByVendor'                    => __("Filter by Vendor", 'shopwp'),
                'filterByVendorHelp'                => __("Match product vendors. Separate multiple by comma.", 'shopwp'),
                'yotpoDesc'                         => __("Add product reviews to ShopWP Pro with the official Yotpo Reviews extension.", 'shopwp'),
                'webhooksDesc'                      => __("Build custom functionality for ShopWP with this powerful Shopify webhooks extension.", 'shopwp'),
                'translatorDesc'                    => __("A full currency and language switcher for ShopWP.", 'shopwp'),
                'rechargeDesc'                      => __("Add subscription products to ShopWP Pro with the official Recharge integration.", 'shopwp'),
                'elementorDesc'                     => __("Add native Elementor functionality to ShopWP Pro with custom widgets.", 'shopwp'),
                'bbDesc'                            => __("Add native Beaver Builder functionality to ShopWP Pro with custom modules.", 'shopwp'),
                'emptyLicense'                      => __('ShopWP error: Oops, it looks like your license key is empty. Please enter a valid key and try again.', 'shopwp'),
                'missingDomain'                     => __('ShopWP error: That domain doesn\'t exist. Double check that you correctly entered your myshopify.com domain', 'shopwp'),
                'badSort'                           => __("ShopWP warning: You must filter products by a Collection first", 'shopwp'),
                'confirmDisconnect'                 => __('Are you sure you want to disconnect your Shopify store?', 'shopwp'),
                'noProdId'                          => __("ShopWP warning: No Shopify product ID found. Try re-syncing your products and trying again.", 'shopwp'),
                'blockProductsDesc'                 => __('This block displays one or more Shopify products in a grid layout.', 'shopwp'),
                'blockProductDesc'                  => __('This block displays a single Shopify product.', 'shopwp'),
                'blockBuyButtonDesc'                => __('This block displays a simple "buy button" without the product title, images, or price.', 'shopwp'),
                'finishedDisconnect'                => __('Successfully finished disconnecting your Shopify store', 'shopwp'),
                'finishedDeLicense'                 => __('Successfully deactivated license key.', 'shopwp'),
                'finishedAcLicense'                 => __('Successfully activated license key.', 'shopwp'),
                'acAPIKey'                          => __('Successfully activated API key.', 'shopwp'),
                'deAPIKey'                          => __('Successfully deactivated API key.', 'shopwp'),
                'disconnecting'                     => __('Disconnecting ...', 'shopwp'),
                'freeTrial'                         => __("Start a 14-day Shopify free trial when you sign-up using our link below. We'll be credited with a referral which helps us continue working on ShopWP!", 'shopwp'),
                'genApiKey'                         => __("Start by generating an API key", 'shopwp'),
                'customImgSize'                     => __("Sets a custom size for the image. Maximum size of 5760 x 5760. Both the height and width values will maintain the image aspect ratio. Therefore, if you want to force all images to the same dimensions make sure to specify the crop option below as well. If you want to size by one dimension only, keep the other dimension set to auto.", 'shopwp'),
                'settingsSaved'                     => __("Settings were saved successfully", 'shopwp'),
                'loadCartHelp'                      => __('Determines whether to use the built-in cart. It\'s recommended to turn this off if you\'re linking products directly to Shopify.', 'shopwp'),
                'findingDomain'                     => __('Your Shopify domain can be found inside the Shopify admin. It will be the domain that\'s displayed in the address bar. (See screenshot below).', 'shopwp'),
                'addedMetafield'                    => __('Successfully added metafield to ShopWP', 'shopwp'),
                'emptyMeta'                         => __('No metafields are currently activated. Click below to start.', 'shopwp'),
                'small'                             => __('Small', 'shopwp'),
                'medium'                            => __('Medium', 'shopwp'),
                'large'                             => __('Large', 'shopwp'),
                "pink"                              => __('Pink', 'shopwp'),
                "red"                               => __('Red', 'shopwp'),
                "orange"                            => __('Orange', 'shopwp'),
                "yellow"                            => __('Yellow', 'shopwp'),
                "teal"                              => __('Teal', 'shopwp'),
                "green"                             => __('Green', 'shopwp'),
                "lightBlue"                         => __('Light Blue', 'shopwp'),
                "blue"                              => __('Blue', 'shopwp'),
                "purple"                            => __('Purple', 'shopwp'),
                "grey"                              => __('Grey', 'shopwp'),
                "palePink"                          => __('Pale pink', 'shopwp'),
                "vividRed"                          => __('Vivid red', 'shopwp'),
                "luminousVivid"                     => __('Luminous vivid', 'shopwp'),
                "luminousVividAmber"                => __('Luminous vivid amber', 'shopwp'),
                "lightGreenCyan"                    => __('Light green', 'shopwp'),
                "vividGreenCyan"                    => __('Vivid green', 'shopwp'),
                "paleCyanBlue"                      => __('Pale cyan', 'shopwp'),
                "cyanBlue"                          => __('Cyan blue', 'shopwp'),
                "lightGrey"                         => __('Light grey', 'shopwp'),
                "mediumBlueGrey"                    => __('Medium blue', 'shopwp'),
                "darkGrey"                          => __('Dark grey', 'shopwp'),
                "addCartColor"                      => __('Add to cart color:', 'shopwp'),
                "addCartText"                       => __('Add to cart button text', 'shopwp'),
                "alignHeight"                       => __('Align height?', 'shopwp'),
                "availSaleHelp"                     => __('Determines whether to show products that are in stock or not', 'shopwp'),
                "any"                               => __('Any', 'shopwp'),
                "productsUrl"                       => __('Products URL', 'shopwp'),
                "productsUrlHelp"                   => __('Sets the permalink of your product detail pages', 'shopwp'),
                'sortHelp'                          => __('Note: sorting by price will take all variant prices into consideration', 'shopwp'),
                'extDescription'                    => __('These ShopWP extensions give your store the functionality it needs. All extensions are sold separately and require', 'shopwp'),
                'extAvail'                          => __('Extensions available for', 'shopwp'),
                'licenseHelp'                       => __('You can find the key within your ShopWP account or your payment confirmation email.', 'shopwp'),
                'yourLicense'                       => __('Your license key:', 'shopwp'),
                'enterLicense'                      => __('Enter your license key below:', 'shopwp'),
                'shopwpLicense'                     => __('ShopWP License key', 'shopwp'),
                'viewExt'                           => __('View this extension', 'shopwp'),
                'needStore'                         => __('Still need a Shopify store?', 'shopwp'),
                'signUpFreeTrial'                   => __('Sign up for a Shopify free trial', 'shopwp'),
                'follow'                            => __('Follow us', 'shopwp'),
                'deLicense'                         => __('Deactivate License', 'shopwp'),
                'acLicense'                         => __('Activate License', 'shopwp'),
                'usingTools'                        => __('Using the Plugin tools', 'shopwp'),
                'demoSite'                          => __('Demo site', 'shopwp'),
                'exampleSites'                      => __('Example sites', 'shopwp'),
                'videoTuts'                         => __('Video Tutorials', 'shopwp'),
                'whatIsSWP'                         => __('What is ShopWP?', 'shopwp'),
                'howSync'                           => __('How to sync your Shopify store', 'shopwp'),
                'howDisplay'                        => __('How to display your products', 'shopwp'),
                'commonIssues'                      => __('Common issues', 'shopwp'),
                'usingShortcodes'                   => __('Using the Products shortcode', 'shopwp'),
                'doc'                               => __('Documentation', 'shopwp'),
                'none'                              => __('None', 'shopwp'),
                'top'                               => __('Top', 'shopwp'),
                'center'                            => __('Center', 'shopwp'),
                'bottom'                            => __('Bottom', 'shopwp'),
                'left'                              => __('Left', 'shopwp'),
                'right'                             => __('Right', 'shopwp'),
                'modal'                             => __('Modal', 'shopwp'),
                'dropdown'                          => __('Dropdown', 'shopwp'),
                'buttons'                           => __('Buttons', 'shopwp'),
                '2'                                 => __('2', 'shopwp'),
                '3'                                 => __('3', 'shopwp'),
                'variantsStyle'                     => __('Variant style', 'shopwp'),
                'searchByHelp'                      => __('When using the search component, this determines what the user searches for. Default is Title.', 'shopwp'),
                'searchBy'                          => __('Search by', 'shopwp'),
                'showFixedTab'                      => __('Show fixed cart tab on', 'shopwp'),
                'showFixedTabDesc'                  => __('Determines when to load the fixed cart tab. Default is all pages.', 'shopwp'),
                'noDataConnection'                  => __("Whoops, no data was returned after saving the connection. Close the window and try again.", 'shopwp'),
                'pricingHelp'                       => __('These settings determine which currency and language to use for product content. All countries and languages are managed inside your Shopify Markets settings. The currency is automatically determined by which country you select, and cannot be changed manually. ', 'shopwp'),
                'showCompareAt'                     => __('Show compare at pricing', 'shopwp'),
                'showCompareAtHelp'                 => __('Show compare at pricingWhen enabled, all products will show the \'compare at\' price if one exists.', 'shopwp'),
                'imageScale'                        => __('Image scale', 'shopwp'),
                'imageScaleHelp'                    => __('Sets a custom scale for all product images. The number here will be multiplied by the dimensions set above. For example, an image originally 450x450 will return an image 900x900 pixels. Will only scale up if the original image is large enough. If original image is too small, the closest image in size will be returned.', 'shopwp'),

                'enableCustomDomainHelp'            => __('When turned on, the plugin will attempt to use the custom domain you have set within Shopify for checkout instead of myshopify.com', 'shopwp'),
                'cropHelp'                          => __('If the entire image won\'t fit in your set dimensions, the crop parameter specifies what part of the image to show.', 'shopwp'),
                'crop'                              => __('Crop', 'shopwp'),
                'enableCustomDomain'                => __('Enable custom checkout domain', 'shopwp'),
                'imageCropPosHelp'                  => __('If the entire image won\'t fit in your set dimensions, the crop parameter specifies what part of the image to show.', 'shopwp'),
                'enableDiscountCode'                => __('Enable discount codes', 'shopwp'),
                'enableDiscountCodeHelp'            => __('Adds an input field to the cart so customers can enter a discount code.', 'shopwp'),
                'showCartTab'                       => __('Show fixed cart tab', 'shopwp'),
                'showCartTabHelp'                   => __('When turned on, will show a fixed cart tab on the side of every page.', 'shopwp'),
                'enableDirectCheckout'              => __('Enable direct checkout', 'shopwp'),
                'enableDirectCheckoutHelp'          => __('When turned on, add to cart buttons will instead take users directly to the checkout.', 'shopwp'),
                'showDefaultPageHeading'            => __('Show Default Page Heading?', 'shopwp'),
                'showDefaultPageHeadingHelp'        => __('Hide / show the heading found on the collections listing page.', 'shopwp'),
                'clpHelp'                           => __('Your collections will appear on this page automatically.', 'shopwp'),
                'clp'                               => __('Collections Listing Page', 'shopwp'),
                'connTo'                            => __('You\'re connected to:', 'shopwp'),
                'enterDomain'                       => __('Enter .myshopify.com domain', 'shopwp'),
                'enablePLP'                         => __('Enable Listing Pages?', 'shopwp'),
                'prodsPerPage'                      => __('Products Per Page', 'shopwp'),
                'plp'                               => __('Products Listing Page', 'shopwp'),
                'plpHelp'                           => __('Your products will appear on this page automatically.', 'shopwp'),
                'pLinkTarget'                       => __('Product Links Open In', 'shopwp'),
                'pLinkTargetHelp'                   => __('Only applicable if product link not set to \'None\'', 'shopwp'),
                'pLinkTo'                           => __('Products Link To', 'shopwp'),
                'pLinkToHelp'                       => __('When set to None, product detail pages are not used.', 'shopwp'),
                'openCheckoutIn'                    => __('Open checkout page in', 'shopwp'),
                'syncingQ'                          => __('Which Shopify data do you want to sync?', 'shopwp'),
                'syncingOptions'                    => __('Syncing options', 'shopwp'),
                'autoSyncing'                       => __('Automatic syncing', 'shopwp'),
                'performance'                       => __('Performance', 'shopwp'),
                'emailUs'                           => __('If you\'re having trouble, please send us an email at: ', 'shopwp'),
                'startAcMeta'                       => __('Start activating metafields!', 'shopwp'),
                'addMeta'                           => __('+ Add metafield', 'shopwp'),
                'imageSizing'                       => __('Custom image sizing', 'shopwp'),
                'imageCropPos'                      => __('Image crop position', 'shopwp'),
                'collectionImg'                     => __('Collection Image', 'shopwp'),
                'variantsTitle'                     => __('Variants Title', 'shopwp'),
                'variantDropdowns'                  => __('Variant dropdowns', 'shopwp'),
                'variantDropdownsBtnColor'          => __('Variant dropdown button color', 'shopwp'),
                'domainFormat'                      => __('By default, your domain uses the format: ', 'shopwp'),
                'height'                            => __('Height', 'shopwp'),
                'width'                             => __('Width', 'shopwp'),
                'scale'                             => __('Scale', 'shopwp'),
                'pages'                             => __('Pages', 'shopwp'),
                'linking'                           => __('Linking', 'shopwp'),
                'collectionsUrl'                    => __('Collections URL', 'shopwp'),
                'collectionsUrlHelp'                => __('Sets the permalink of your collection detail pages', 'shopwp'),
                'plugin'                            => __('Plugin', 'shopwp'),
                'metafields'                        => __('Metafields', 'shopwp'),
                'metafieldNS'                       => __('Metafield Namespace', 'shopwp'),
                'metafieldKey'                      => __('Metafield Key', 'shopwp'),
                'key'                               => __('Key', 'shopwp'),
                'namespace'                         => __('Namespace', 'shopwp'),
                'de'                                => __('Deactivate', 'shopwp'),
                'ac'                                => __('Activate', 'shopwp'),
                'enterKey'                          => __('Enter your api key here', 'shopwp'),
                'enterSecretKey'                    => __('Enter your api secret key here', 'shopwp'),
                'acOn'                              => __('Activated on', 'shopwp'),
                'deOn'                              => __('Deactivated on', 'shopwp'),
                'currentTab'                        => __('Current tab / window', 'shopwp'),
                'newTab'                            => __('New tab / window', 'shopwp'),
                'savingSettings'                    => __('Saving Settings', 'shopwp'),
                'saveSettings'                      => __('Save ShopWP Settings', 'shopwp'),
                'numPixels'                         => __('Number in pixels or auto.', 'shopwp'),
                'min2'                              => __('2 minutes', 'shopwp'),
                'min5'                              => __('5 minutes', 'shopwp'),
                'min20'                             => __('20 minutes', 'shopwp'),
                'hr1'                               => __('1 hour', 'shopwp'),
                'hr12'                              => __('12 hours', 'shopwp'),
                'd1'                                => __('1 day', 'shopwp'),
                'w2'                                => __('2 weeks', 'shopwp'),
                'm1'                                => __('1 month', 'shopwp'),
                'cache'                             => __('Cache', 'shopwp'),
                'cacheLength'                       => __('Cache length', 'shopwp'),
                'enableCache'                       => __('Enable data cache', 'shopwp'),
                'checkoutBtnBg'                     => __('Checkout Button Background', 'shopwp'),
                'paginationBtnBg'                   => __('Pagination Button Background', 'shopwp'),
                'manualPages'                       => __('Manually selected pages', 'shopwp'),
                'onlyWithProds'                     => __('Only pages with products', 'shopwp'),
                'allPages'                          => __('All pages', 'shopwp'),
                'fixedCounterBgColor'               => __('Fixed cart counter background color', 'shopwp'),
                'fixedCounterTextColor'             => __('Fixed cart counter text color', 'shopwp'),
                'fixedIconBgColor'                  => __('Fixed cart icon background color', 'shopwp'),
                'fixedIconColor'                    => __('Fixed cart icon color', 'shopwp'),
                'loadCart'                          => __('Load cart', 'shopwp'),
                'enableCartNotes'                   => __('Enable cart notes', 'shopwp'),
                'enableCartTerms'                   => __('Enable cart terms', 'shopwp'),
                'syncing'                           => __('Syncing', 'shopwp'),
                'alignHeight'                       => __('Align products height', 'shopwp'),
                'hidePag'                           => __('Hide all pagination', 'shopwp'),
                'showCrumbs'                        => __('Show breadcrumbs', 'shopwp'),
                'usageData'                         => __('Share Usage Data', 'shopwp'),
                'betaUpdates'                       => __('Enable Beta Updates', 'shopwp'),
                'featImg'                           => __('Featured Images', 'shopwp'),
                'thumbs'                            => __('Thumbnail Images', 'shopwp'),
                'showTax'                           => __('Show estimated tax?', 'shopwp'),
                'showTaxHelp'                       => __('Adds an estimated tax above the subtotal.', 'shopwp'),
                'template'                          => __('Template', 'shopwp'),
                'templateHelp'                      => __('Allows you to customize the layout of the products using HTML.', 'shopwp'),
                'created'                           => __('Created', 'shopwp'),
            ]
        ];
    }

    public function stringify_settings($settings)
    {
        $settings_encoded_js_string = wp_json_encode(
            Utils::convert_underscore_to_camel_array($settings)
        );

        return "var shopwp = " . $settings_encoded_js_string;
    }

    public function add_settings_script($script_dep, $is_admin)
    {
        $string_settings = $this->stringify_settings($this->get_settings($is_admin));

        wp_add_inline_script($script_dep, $string_settings, 'before');
    }
}
