<?php

namespace ShopWP;

use ShopWP\Utils;
use ShopWP\Options;
use ShopWP\Utils\Data;
use ShopWP\Utils\Server;

if (!defined('ABSPATH')) {
    exit();
}

class Frontend
{
    public $plugin_settings;
    public $Data_Bridge;

    public function __construct($plugin_settings, $Data_Bridge)
    {
        $this->plugin_settings = $plugin_settings;
        $this->Data_Bridge = $Data_Bridge;
    }

    public function public_scripts()
    {

        global $post;

        $hash = '.c672079a';
        $runtime_url            = SHOPWP_PLUGIN_URL . 'dist/runtime' . $hash . '.js';
        $vendors_admin_url      = SHOPWP_PLUGIN_URL . 'dist/vendors-public' . $hash . '.js';
        $main_url               = SHOPWP_PLUGIN_URL . 'dist/public' . $hash . '.js';

        wp_register_script('shopwp-runtime', $runtime_url, ['wp-hooks', 'wp-element'], false, true);
        wp_register_script('shopwp-vendors-public', $vendors_admin_url, ['wp-hooks', 'wp-element'], false, true);
        wp_register_script(
            'shopwp-public', 
            $main_url, 
            [
                'wp-hooks',
                'wp-element',
                'shopwp-runtime',
                'shopwp-vendors-public',
            ],
            false,
            true
        );

        if (empty($this->plugin_settings['shop'])) {
            $currency = 'USD';
        } else {
            $currency = $this->plugin_settings['shop']['currency'];
        }

        // Global plugin JS settings
        $this->Data_Bridge->add_settings_script('shopwp-public', false);

        wp_enqueue_script('shopwp-runtime');
        wp_enqueue_script('shopwp-vendors-public');
        wp_enqueue_script('shopwp-public');
        
    }


    public function defer_parsing_of_js($tag, $handle) {
        
        if ( 'shopwp-public' !== $handle ) {
            return $tag;
        }
        
        return str_replace( ' src', ' defer="defer" src', $tag );
    }


    public function css_body_class($classes)
    {
        $classes[] = 'shopwp';

        return $classes;
    }

    /*
    
    This fixes thew infamous "Cookie check failed" error documented here: https://docs.wpshop.io/guides/common-issues#cookie-check-failed
    Added: 5/5/23
    
    */
    public function sanitize_nonce($errors) {

        // Bail if rest_route isn't defined (shouldn't happen!)
        if ( empty( $GLOBALS['wp']->query_vars['rest_route'] ) ) {
            return $errors;
        }

        $route = ltrim( $GLOBALS['wp']->query_vars['rest_route'], '/' );

        // Ensure we're dealing with our REST request only
        if ( strpos( $route, 'shopwp/v1' ) !== 0 ) {
            return $errors;
        }

        if ( !empty( $_SERVER['HTTP_X_WP_NONCE'] ) ) {
            $nonce = $_SERVER['HTTP_X_WP_NONCE'];

            if ( !\wp_verify_nonce($nonce, 'wp_rest') ) {
                // Nonce check failed, so create a new one.
                $_SERVER['HTTP_X_WP_NONCE'] = \wp_create_nonce('wp_rest');
            }
        }

        return $errors;
    }

    public function init()
    {
        if (\is_admin()) {
            return;
        }

        add_filter('body_class', [$this, 'css_body_class']);
        add_action('wp_enqueue_scripts', [$this, 'public_scripts']);
        add_filter('script_loader_tag', [$this, 'defer_parsing_of_js'], 10, 2 );
        add_filter('rest_authentication_errors', [$this, 'sanitize_nonce']);
    }
}