<?php

namespace ShopWP\Processing;

if (!defined('ABSPATH')) {
    exit();
}

use ShopWP\Utils;
use ShopWP\Utils\Server;

class Posts_Collections extends \ShopWP\Processing\Vendor_Background_Process
{
    protected $action = 'shopwp_background_processing_posts_collections';

    public $DB_Settings_Syncing;
    public $CPT_Model;

    public function __construct(
        $DB_Settings_Syncing,
        $CPT_Model
    ) {
        $this->DB_Settings_Syncing = $DB_Settings_Syncing;
        $this->CPT_Model = $CPT_Model;

        parent::__construct($DB_Settings_Syncing);
    }

    public function process($items, $params = false)
    {

        if ($this->DB_Settings_Syncing->expired_from_server_issues($items, __METHOD__, __LINE__)) {
            parent::complete();
            return;
        }

        $this->dispatch_items($items);
    }


    protected function task($collection)
    {

        // Stops background process if syncing stops
        if (!$this->DB_Settings_Syncing->is_syncing()) {
            parent::complete();
            return false;
        }

        if ($this->time_exceeded() || $this->memory_exceeded()) {
            return $collection;
        }

        $collection_id = Utils::convert_storefront_id_to_numeric($collection->id, 'Collection');


        if (empty($post_id)) {
            $post_id = false;

        } else {
            $post_id = $post_id[0];
        }

        $result = $this->CPT_Model->insert_or_update_collection_post(
            $collection,
            $collection_id,
            $post_id
        );

        if (is_wp_error($result)) {
            $this->DB_Settings_Syncing->save_notice_and_expire_sync($result);
            parent::complete();
            return false;
        }

        return false;
    }

    protected function after_queue_item_removal($item)
    {
        $this->DB_Settings_Syncing->increment_current_amount(
            'smart_collections'
        );
    }
}