import yargs from "yargs"
import { hideBin } from "yargs/helpers"

import browserSync from "browser-sync"
import {
  isBuilding,
  getRelease,
  getCurrent,
  getCommitMsg,
} from "./gulp/utils/utils.js"

const argvs = yargs(hideBin(process.argv)).argv

const config = {
  files: {
    proZip: "./_pro/shopwp-pro-" + getRelease(argvs) + ".zip",
    proReadme: "./_pro/README.txt",
    entryTmp: "./_tmp/shopwp.php",
    versionLocations: ["./_tmp/shopwp.php", "./_tmp/classes/class-config.php"],
    versionLocationsReal: ["./shopwp.php", "./classes/class-config.php"],
    versionDocs: ["/Users/andrew/_www/shopwp-docs/docusaurus.config.js"],
    toBeProcessedTmp: [
      "./_tmp/admin/js/**/*",
      "./_tmp/admin/partials/**/*",
      "./_tmp/public/js/**/*",
      "./_tmp/public/templates/**/*",
      "./_tmp/classes/**/*",
      "./_tmp/shopwp.php",
      "./_tmp/uninstall.php",
    ],
    npmPackageJSONs: [
      "./npm-packages/@shopwp/api/package.json",
      "./npm-packages/@shopwp/blocks/package.json",
      "./npm-packages/@shopwp/common/package.json",
      "./npm-packages/@shopwp/components/package.json",
      "./npm-packages/@shopwp/hooks/package.json",
      "./npm-packages/@shopwp/wp-ui/package.json",
      "./package.json",
    ],
    tmpVendor: "./_tmp/vendor/**/*",
    pluginTitleSettings: ["./_tmp/admin/partials/display.php"],
    pluginUpdateFunction: ["./_tmp/classes/class-hooks.php"],
    tmp: "./_tmp/**/*",
    tmpJS: ["./_tmp/public/js/**/*", "./_tmp/admin/js/**/*"],
    tmpAll: ["./_tmp/**/*", "./_tmp/**/.*"],
    test: "hello/*",
    all: [
      "./**/*",
      "!./dist/**/*",
      "!./yarn.lock",
      "!**/node_modules/**",
      "!./.vscode/**/*",
      "!./bin/**",
      "!./.git/**",
      "!./tests/**",
      "!**/__tests__/**",
      "!**/__mocks__/**",
      "!./gulp/**",
      "!./stats.html",
      "!./.travis.yml",
      "!./_free",
      "!./_tmp",
      "!./_pro",
      "!./shopwp-pro",
      "!./.eslintrc",
      "!./**/*.DS_Store",
      "!./**/*.babelrc",
      "!./package.json",
      "!./phpunit.xml.dist",
      "!./postcss.config.js",
      "!./gulpfile.babel.js",
    ],

    // Represents all the files needed for other developers to work with. What gets committed to the free repo.
    onlyWorking: [
      "./**/*",
      "./**/.*",
      "./**/.**/**",
      "!./node_modules/**",
      "!./.git/**",
      "!./_tmp/**",
      "!./_free/**",
      "!./.gitignore",
    ],
    build: "./**/*",
    buildProContent: "../../../../assets/shopwp-pro/**/*",
    distProFiles: "../../../../assets/shopwp-pro/**/*",
    tmpProClear: [
      "./_tmp/.git/**",
      "./_tmp/.gitignore",
      "./_tmp/.eslintrc",
      "./_tmp/.browserslistrc",
      "./_tmp/.travis.yml",
      "./_tmp/travis-php.ini",
      "./_tmp/.phpunit-watcher.yml",
      "./_tmp/.vscode/**/*",
      "./_tmp/admin/js",
      "./_tmp/public/js",
      "./_tmp/admin/css/app",
      "./_tmp/public/css/app",
      "./_tmp/**/*.DS_Store",
      "./_tmp/jest-test-env.js",
      "./_tmp/babel.config.js",
      "./_tmp/jest-test-setup.js",
      "./_tmp/**/*.babelrc",
      "./_tmp/yarn.lock",
      "./_tmp/node_modules/**",
      "./_tmp/npm-packages",
      "./_tmp/bin/**",
      "./_tmp/tests/**",
      "./_tmp/gulp/**",
      "./_tmp/stats.html",
      "./_tmp/_free",
      "./_tmp/_tmp",
      "./_tmp/_pro",
      "./_tmp/package.json",
      "./_tmp/package-lock.json",
      "./_tmp/phpunit.xml.dist",
      "./_tmp/postcss.config.js",
      "./_tmp/gulpfile.babel.js",
      "./_tmp/webpack.config.babel.js",
      "./_tmp/composer.json",
      "./_tmp/composer.lock",
      "./_tmp/babel.config.json",
    ],
    tmpFinalZip: ["./_tmp/shopwp-pro-" + getRelease(argvs) + ".zip"],
    tmpFinalFinals: ["./_tmp/shopwp-pro/**/*"],
    buildEntry: ["./admin/js/app/tools/tools.js"],
    js: [
      "./admin/**/*.js",
      "./public/**/*.js",
      "./npm-packages/**/*.js",
      "!**/node_modules/**",
    ],
    jsPublic: [
      // doesnt need tmp check
      "./public/js/app/**/*.js",
      "./public/js/app/**/*.jsx",
      "!./public/js/app.min.js",
      "!./public/js/vendor.min.js",
      "!./public/js/app.min.js.map",
    ],
    jsAdmin: [
      // doesnt need tmp check
      "./admin/js/app/**/*.js",
      "./admin/js/app/**/*.jsx",
      "./admin/js/blocks/**/*.js",
      "./admin/js/blocks/**/*.jsx",
      "./admin/js/wizard/**/*.js",
      "./admin/js/wizard/**/*.jsx",
      "!./admin/js/app.min.js",
      "!./admin/js/vendor.min.js",
      "!./admin/js/app.min.js.map",
    ],
    freeRepoFiles: "./_free/*",
    freeRepoFilesAll: [
      "./_free/**/*",
      "./_free/**/.*",
      "./_free/.**/*",
      "./_free/.**/.*",
    ],
    proRepoFiles: "./*",
    jsEntryPublic: isBuilding(argvs)
      ? "./_tmp/public/js/app/app.js"
      : "./public/js/app/app.js",
    jsEntryAdmin: isBuilding(argvs)
      ? "./_tmp/admin/js/app/app.js"
      : "./admin/js/app/app.js",
    cssAdmin: "./admin/css/**/*.scss", // doesnt need tmp check
    cssEntryAdmin: isBuilding(argvs)
      ? "./_tmp/admin/css/app/app.scss"
      : "./admin/css/app/app.scss",
    svgsPublic: isBuilding(argvs)
      ? "./_tmp/public/imgs/**/*.svg"
      : "./public/imgs/**/*.svg",
    svgsAdmin: isBuilding(argvs)
      ? "./_tmp/admin/imgs/**/*.svg"
      : "./admin/imgs/**/*.svg",
    assetFiles: isBuilding(argvs)
      ? [
          "./_tmp/classes/class-backend.php",
          "./_tmp/classes/class-frontend.php",
        ]
      : ["./classes/class-backend.php", "./classes/class-frontend.php"],
    distFilesSafeToRemove: isBuilding(argvs)
      ? [
          "./_tmp/dist/**/*.br",
          "./_tmp/dist/**/*.br",
          "./_tmp/dist/**/*.css",
          "!./_tmp/dist",
        ]
      : ["./dist/**/*.js", "./dist/**/*.br", "./dist/**/*.css", "!./dist"],
  },
  folders: {
    tmp: "./_tmp",
    allTmp: ["./_tmp", "./_pro"],
    tmpPro: "./shopwp-pro/**/*",
    tmpFree: "./shopwp/**/*",
    freeRepo: "./_free",
    plugin: "./",
    root: isBuilding(argvs) ? "./_tmp" : "./",
    dist: isBuilding(argvs) ? "./_tmp/dist" : "./dist",
    distJS: "./dist/**/*.js",
    pro: "../../../../assets/shopwp-pro",
    proTmp: "../../../../assets/shopwp-pro/_tmp",
    proTmpRenamed: "../../../../assets/shopwp-pro/shopwp-pro",
    free: "../../../../assets/shopwp",
    freeDistRepo: "../../../../assets/shopwp/_free",
    freeTmpRenamed: "../../../../assets/shopwp/shopwp",
    svgsPublic: isBuilding(argvs) ? "./_tmp/public/imgs" : "./public/imgs",
    svgsAdmin: isBuilding(argvs) ? "./_tmp/admin/imgs" : "./admin/imgs",
    cache: "./node_modules/.cache",
    docs: "/Users/andrew/_www/shopwp-docs",
    shopwpApi: "./npm-packages/@shopwp/api",
    shopwpBlocks: "./npm-packages/@shopwp/blocks",
    shopwpCommon: "./npm-packages/@shopwp/common",
    shopwpComponents: "./npm-packages/@shopwp/components",
    shopwpHooks: "./npm-packages/@shopwp/hooks",
    shopwpUI: "./npm-packages/@shopwp/wp-ui",
  },
  names: {
    jsPublic: "public.min.js",
    jsAdmin: "admin.min.js",
    cssAdmin: "admin.min.css",
    pro: "ShopWP Pro",
    free: "ShopWP",
    packages: {
      prod: {
        blocks: "'Blocks'",
        wizard: "'@shopwp/wizard'",
        components: "'Components'",
        api: "'Api'",
      },
    },
    zips: {
      pro: "shopwp-pro-" + getRelease(argvs) + ".zip",
      free: "shopwp.zip",
    },
  },

  replacementPatterns: [
    /(?<=dist\/admin\.).*?(?=\.js)/g,
    /(?<=dist\/public\.).*?(?=\.js)/g,
    /(?<=dist\/vendors-public\.).*?(?=\.js)/g,
    /(?<=dist\/vendors-admin-public\.).*?(?=\.js)/g,
    /(?<=dist\/vendors-admin\.).*?(?=\.js)/g,
    /(?<=dist\/runtime\.).*?(?=\.js)/g,
    /(?<=dist\/blocks\.).*?(?=\.js)/g,
    /(?<=dist\/vendors-blocks\.).*?(?=\.js)/g,
    /(?<=dist\/commons\.).*?(?=\.js)/g,
    /(?<=dist\/wizard\.).*?(?=\.js)/g,
  ],
  bs: browserSync.create(),
  serverName: "shopwp.loc",
  isBuilding: isBuilding(argvs),
  isFree: false,
  isPro: true,
  buildTier: "pro",
  buildRelease: getRelease(argvs), // Plugin version number
  currentRelease: getCurrent(argvs), // Current Plugin version number
  commitMsg: getCommitMsg(argvs), // Plugin version number
}

export default config
